@@ -27,7 +27,6 @@ sys.modules['gobject'] = GObject
 
 import dbus.mainloop.glib
 import slip.dbus
-from dbus.exceptions import DBusException
 
 from firewall.config import *
 from firewall.config.dbus import *
@@ -41,21 +40,27 @@ exception_handler = None
 def handle_exceptions(func, *args, **kwargs):
     """Decorator to handle exceptions
     """
-    while 1:
-        try:
-            return func(*args, **kwargs)
-        except DBusException as e:
-            if not exception_handler:
-                raise
-            if "NotAuthorizedException" in str(e):
-                exception_handler(_("Authorization failed."))
+    global exception_handler
+    #while 1:
+    try:
+        return func(*args, **kwargs)
+    except dbus.exceptions.DBusException as e:
+        dbus_message = e.get_dbus_message()
+        dbus_name = e.get_dbus_name()
+        if not exception_handler:
+            raise
+        if "NotAuthorizedException" in dbus_name:
+            exception_handler(_("Authorization failed."))
+        else:
+            if dbus_message:
+                exception_handler(dbus_message)
             else:
-                exception_handler(e)
-        except Exception as msg:
-            if not exception_handler:
-                raise
-            else:
-                exception_handler(e)
+                exception_handler(str(e))
+    except Exception as e:
+        if not exception_handler:
+            raise
+        else:
+            exception_handler(str(e))
 
 # zone config setings
 
@@ -65,7 +70,8 @@ class FirewallClientConfigZoneSettings(object):
         if settings:
             self.settings = settings
         else:
-            self.settings = ["", "", "", False, "", [], [], [], False, []]
+            self.settings = ["", "", "", False, "", [], [], [], False, [],
+                             [], [], []]
 
     @handle_exceptions
     def getVersion(self):
@@ -88,12 +94,7 @@ class FirewallClientConfigZoneSettings(object):
     def setDescription(self, description):
         self.settings[2] = description
 
-    @handle_exceptions
-    def getImmutable(self):
-        return self.settings[3]
-    @handle_exceptions
-    def setImmutable(self, immutable):
-        self.settings[3] = immutable
+    # self.settings[3] was used for 'immutable'
 
     @handle_exceptions
     def getTarget(self):
@@ -153,7 +154,7 @@ class FirewallClientConfigZoneSettings(object):
         if icmptype in self.settings[7]:
             self.settings[7].remove(icmptype)
     @handle_exceptions
-    def querayIcmpBlock(self, icmptype):
+    def queryIcmpBlock(self, icmptype):
         return icmptype in self.settings[7]
 
     @handle_exceptions
@@ -193,6 +194,61 @@ class FirewallClientConfigZoneSettings(object):
             to_addr = ''
         return (port,protocol,to_port,to_addr) in self.settings[9]
 
+    @handle_exceptions
+    def getInterfaces(self):
+        return self.settings[10]
+    @handle_exceptions
+    def setInterfaces(self, interfaces):
+        self.settings[10] = interfaces
+    @handle_exceptions
+    def addInterface(self, interface):
+        if interface not in self.settings[10]:
+            self.settings[10].append(interface)
+    @handle_exceptions
+    def removeInterface(self, interface):
+        if interface in self.settings[10]:
+            self.settings[10].remove(interface)
+    @handle_exceptions
+    def queryInterface(self, interface):
+        return interface in self.settings[10]
+
+    @handle_exceptions
+    def getSources(self):
+        return self.settings[11]
+    @handle_exceptions
+    def setSources(self, sources):
+        self.settings[11] = sources
+    @handle_exceptions
+    def addSource(self, source):
+        if source not in self.settings[11]:
+            self.settings[11].append(source)
+    @handle_exceptions
+    def removeSource(self, source):
+        if source in self.settings[11]:
+            self.settings[11].remove(source)
+    @handle_exceptions
+    def querySource(self, source):
+        return source in self.settings[11]
+
+    @handle_exceptions
+    def getRichRules(self):
+        return self.settings[12]
+    @handle_exceptions
+    def setRichRules(self, rules):
+        self.settings[12] = rules
+    @handle_exceptions
+    def addRichRule(self, rule):
+        if rule not in self.settings[12]:
+            self.settings[12].append(rule)
+    @handle_exceptions
+    def removeRichRule(self, rule):
+        if rule in self.settings[12]:
+            self.settings[12].remove(rule)
+    @handle_exceptions
+    def queryRichRule(self, rule):
+        return rule in self.settings[12]
+
+
 # zone config
 
 class FirewallClientConfigZone(object):
@@ -251,7 +307,7 @@ class FirewallClientConfigZone(object):
 
 # service config settings
 
-class FirewallClientConfigServiceSettings(object):
+class FirewallClientServiceSettings(object):
     @handle_exceptions
     def __init__(self, settings=None):
         if settings:
@@ -366,7 +422,7 @@ class FirewallClientConfigService(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def getSettings(self):
-        return FirewallClientConfigServiceSettings(list(dbus_to_python(\
+        return FirewallClientServiceSettings(list(dbus_to_python(\
                     self.fw_service.getSettings())))
 
     @slip.dbus.polkit.enable_proxy
@@ -391,7 +447,7 @@ class FirewallClientConfigService(object):
 
 # icmptype config settings
 
-class FirewallClientConfigIcmpTypeSettings(object):
+class FirewallClientIcmpTypeSettings(object):
     @handle_exceptions
     def __init__(self, settings=None):
         if settings:
@@ -470,7 +526,7 @@ class FirewallClientConfigIcmpType(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def getSettings(self):
-        return FirewallClientConfigIcmpTypeSettings(list(dbus_to_python(\
+        return FirewallClientIcmpTypeSettings(list(dbus_to_python(\
                     self.fw_icmptype.getSettings())))
 
     @slip.dbus.polkit.enable_proxy
@@ -493,6 +549,110 @@ class FirewallClientConfigIcmpType(object):
     def rename(self, name):
         self.fw_icmptype.rename(name)
 
+# config.policies lockdown whitelist
+
+class FirewallClientConfigPoliciesLockdownWhitelist(object):
+    @handle_exceptions
+    def __init__(self, settings=None):
+        if settings:
+            self.settings = settings
+        else:
+            self.settings = [ [], [], [], [] ]
+
+    @handle_exceptions
+    def getCommands(self):
+        return self.settings[0]
+    @handle_exceptions
+    def setCommands(self, commands):
+        self.settings[0] = commands
+    @handle_exceptions
+    def addCommand(self, command):
+        if command not in self.settings[0]:
+            self.settings[0].append(command)
+    @handle_exceptions
+    def removeCommand(self, command):
+        if command in self.settings[0]:
+            self.settings[0].remove(command)
+    @handle_exceptions
+    def queryCommand(self, command):
+        return command in self.settings[0]
+
+    @handle_exceptions
+    def getContexts(self):
+        return self.settings[1]
+    @handle_exceptions
+    def setContexts(self, contexts):
+        self.settings[1] = contexts
+    @handle_exceptions
+    def addContext(self, context):
+        if context not in self.settings[1]:
+            self.settings[1].append(context)
+    @handle_exceptions
+    def removeContext(self, context):
+        if context in self.settings[1]:
+            self.settings[1].remove(context)
+    @handle_exceptions
+    def queryContext(self, context):
+        return context in self.settings[1]
+
+    @handle_exceptions
+    def getUsers(self):
+        return self.settings[2]
+    @handle_exceptions
+    def setUsers(self, users):
+        self.settings[2] = users
+    @handle_exceptions
+    def addUser(self, user):
+        if user not in self.settings[2]:
+            self.settings[2].append(user)
+    @handle_exceptions
+    def removeUser(self, user):
+        if user in self.settings[2]:
+            self.settings[2].remove(user)
+    @handle_exceptions
+    def queryUser(self, user):
+        return user in self.settings[2]
+
+    @handle_exceptions
+    def getUids(self):
+        return self.settings[3]
+    @handle_exceptions
+    def setUids(self, uids):
+        self.settings[3] = uids
+    @handle_exceptions
+    def addUid(self, uid):
+        if uid not in self.settings[3]:
+            self.settings[3].append(uid)
+    @handle_exceptions
+    def removeUid(self, uid):
+        if uid in self.settings[3]:
+            self.settings[3].remove(uid)
+    @handle_exceptions
+    def queryUid(self, uid):
+        return uid in self.settings[3]
+
+# config.policies
+
+class FirewallClientConfigPolicies(object):
+    @handle_exceptions
+    def __init__(self, bus):
+        self.bus = bus
+        self.dbus_obj = self.bus.get_object(DBUS_INTERFACE,
+                                            DBUS_PATH_CONFIG)
+        self.fw_policies = dbus.Interface( \
+            self.dbus_obj, dbus_interface=DBUS_INTERFACE_CONFIG_POLICIES)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getLockdownWhitelist(self):
+        return FirewallClientConfigPoliciesLockdownWhitelist( \
+            list(dbus_to_python(self.fw_policies.getLockdownWhitelist())))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def setLockdownWhitelist(self, settings):
+        self.fw_policies.setLockdownWhitelist(tuple(settings.settings))
+
 # config
 
 class FirewallClientConfig(object):
@@ -503,6 +663,28 @@ class FirewallClientConfig(object):
                                             DBUS_PATH_CONFIG)
         self.fw_config = dbus.Interface(self.dbus_obj,
                                         dbus_interface=DBUS_INTERFACE_CONFIG)
+        self.fw_properties = dbus.Interface(
+            self.dbus_obj, dbus_interface='org.freedesktop.DBus.Properties')
+        self._policies = FirewallClientConfigPolicies(self.bus)
+
+    # properties
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def get_property(self, prop):
+        return dbus_to_python(self.fw_properties.Get(DBUS_INTERFACE_CONFIG,
+                                                     prop))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def get_properties(self):
+        return dbus_to_python(self.fw_properties.GetAll(DBUS_INTERFACE_CONFIG))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def set_property(self, prop, value):
+        self.fw_properties.Set(DBUS_INTERFACE_CONFIG, prop, value)
+
     # zone
 
     @slip.dbus.polkit.enable_proxy
@@ -521,6 +703,16 @@ class FirewallClientConfig(object):
         path = dbus_to_python(self.fw_config.getZoneByName(name))
         return FirewallClientConfigZone(self.bus, path)
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getZoneOfInterface(self, iface):
+        return dbus_to_python(self.fw_config.getZoneOfInterface(iface))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getZoneOfSource(self, source):
+        return dbus_to_python(self.fw_config.getZoneOfSource(source))
+
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def addZone(self, name, settings):
@@ -575,11 +767,15 @@ class FirewallClientConfig(object):
         path = self.fw_config.addIcmpType(name, tuple(settings.settings))
         return FirewallClientConfigIcmpType(self.bus, path)
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def policies(self):
+        return self._policies
 #
 
 class FirewallClient(object):
     @handle_exceptions
-    def __init__(self, bus=None, wait=0, quiet=False):
+    def __init__(self, bus=None, wait=0, quiet=True):
         if not bus:
             dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
             try:
@@ -599,10 +795,12 @@ class FirewallClient(object):
         for interface in [ DBUS_INTERFACE,
                            DBUS_INTERFACE_ZONE,
                            DBUS_INTERFACE_DIRECT,
+                           DBUS_INTERFACE_POLICIES,
                            DBUS_INTERFACE_CONFIG,
                            DBUS_INTERFACE_CONFIG_ZONE,
                            DBUS_INTERFACE_CONFIG_SERVICE,
-                           DBUS_INTERFACE_CONFIG_ICMPTYPE ]:
+                           DBUS_INTERFACE_CONFIG_ICMPTYPE,
+                           DBUS_INTERFACE_CONFIG_POLICIES ]:
             self.bus.add_signal_receiver(self._signal_receiver,
                                          dbus_interface=interface,
                                          interface_keyword='interface',
@@ -631,10 +829,35 @@ class FirewallClient(object):
             "forward-port-removed": "ForwardPortRemoved",
             "icmp-block-added": "IcmpBlockAdded",
             "icmp-block-removed": "IcmpBlockRemoved",
+            "richrule-added": "RichRuleAdded",
+            "richrule-removed": "RichRuleRemoved",
             "interface-added": "InterfaceAdded",
             "interface-removed": "InterfaceRemoved",
-            "zone-changed": "ZoneChanged",
+            "zone-changed": "ZoneOfInterfaceChanged", # DEPRECATED, use zone-of-interface-changed instead
+            "zone-of-interface-changed": "ZoneOfInterfaceChanged",
+            "source-added": "SourceAdded",
+            "source-removed": "SourceRemoved",
+            "zone-of-source-changed": "ZoneOfSourceChanged",
+            # direct callbacks
+# TODO
+            "direct-chain-added": "ChainAdded",
+            "direct-chain-removed": "ChainRemoved",
+            "direct-rule-added": "RuleAdded",
+            "direct-rule-removed": "RuleRemoved",
+            # policy callbacks
+# TODO
+            "lockdown-enabled": "LockdownEnabled",
+            "lockdown-disabled": "LockdownDisabled",
+            "lockdown-whitelist-command-added": "LockdownWhitelistCommandAdded",
+            "lockdown-whitelist-command-removed": "LockdownWhitelistCommandRemoved",
+            "lockdown-whitelist-context-added": "LockdownWhitelistContextAdded",
+            "lockdown-whitelist-context-removed": "LockdownWhitelistContextRemoved",
+            "lockdown-whitelist-uid-added": "LockdownWhitelistUidAdded",
+            "lockdown-whitelist-uid-removed": "LockdownWhitelistUidRemoved",
+            "lockdown-whitelist-user-added": "LockdownWhitelistUserAdded",
+            "lockdown-whitelist-user-removed": "LockdownWhitelistUserRemoved",
             # firewalld.config callbacks
+            "config:policies:lockdown-whitelist-updated": "config:policies:LockdownWhitelistUpdated",
             "config:zone-added": "config:ZoneAdded",
             "config:zone-updated": "config:ZoneUpdated",
             "config:zone-removed": "config:ZoneRemoved",
@@ -669,12 +892,22 @@ class FirewallClient(object):
         self._config = None
         self.connected = False
 
+    @handle_exceptions
+    def getExceptionHandler(self):
+        global exception_handler
+        return exception_handler
+
+    @handle_exceptions
+    def setExceptionHandler(self, handler):
+        global exception_handler
+        exception_handler = handler
+
     @handle_exceptions
     def connect(self, name, callback, *args):
         if name in self._callbacks:
             self._callback[self._callbacks[name]] = (callback, args)
         else:
-            raise ValueError, "Unknown callback name '%s'" % name
+            raise ValueError("Unknown callback name '%s'" % name)
 
     @handle_exceptions
     def _dbus_connection_changed(self, name, old_owner, new_owner):
@@ -698,18 +931,19 @@ class FirewallClient(object):
                                           dbus_interface=DBUS_INTERFACE_ZONE)
             self.fw_direct = dbus.Interface(
                 self.dbus_obj, dbus_interface=DBUS_INTERFACE_DIRECT)
+            self.fw_policies = dbus.Interface(
+                self.dbus_obj, dbus_interface=DBUS_INTERFACE_POLICIES)
             self.fw_properties = dbus.Interface(
                 self.dbus_obj, dbus_interface='org.freedesktop.DBus.Properties')
-        except DBusException as e:
+        except dbus.exceptions.DBusException as e:
             # ignore dbus errors
             if not self.quiet:
-                print "DBusException", e
+                print ("DBusException", e.get_dbus_message())
             return
         except Exception as e:
             if not self.quiet:
-                print "Exception", e
+                print ("Exception", e)
             return
-
         self._config = FirewallClientConfig(self.bus)
         self.connected = True
         self._signal_receiver(member="connection-established",
@@ -749,6 +983,8 @@ class FirewallClient(object):
             signal = "config:IcmpType" + signal
         elif interface == DBUS_INTERFACE_CONFIG:
             signal = "config:" + signal
+        elif interface == DBUS_INTERFACE_CONFIG_POLICIES:
+            signal = "config:policies:" + signal
 
         for callback in self._callbacks:
             if self._callbacks[callback] == signal and \
@@ -818,14 +1054,28 @@ class FirewallClient(object):
 
     # list functions
 
+    @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def listServices(self):
         return dbus_to_python(self.fw.listServices())
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getServiceSettings(self, service):
+        return FirewallClientServiceSettings(list(dbus_to_python(\
+                    self.fw.getServiceSettings(service))))
+
+    @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def listIcmpTypes(self):
         return dbus_to_python(self.fw.listIcmpTypes())
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getIcmpTypeSettings(self, icmptype):
+        return FirewallClientIcmpTypeSettings(list(dbus_to_python(\
+                    self.fw.getIcmpTypeSettings(icmptype))))
+
     # default zone
 
     @slip.dbus.polkit.enable_proxy
@@ -855,6 +1105,11 @@ class FirewallClient(object):
     def getZoneOfInterface(self, interface):
         return dbus_to_python(self.fw_zone.getZoneOfInterface(interface))
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getZoneOfSource(self, source):
+        return dbus_to_python(self.fw_zone.getZoneOfSource(source))
+
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def isImmutable(self, zone):
@@ -869,9 +1124,15 @@ class FirewallClient(object):
 
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
-    def changeZone(self, zone, interface):
+    def changeZone(self, zone, interface): # DEPRECATED
         return dbus_to_python(self.fw_zone.changeZone(zone, interface))
 
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def changeZoneOfInterface(self, zone, interface):
+        return dbus_to_python(self.fw_zone.changeZoneOfInterface(zone,
+                                                                 interface))
+
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def getInterfaces(self, zone):
@@ -887,6 +1148,55 @@ class FirewallClient(object):
     def removeInterface(self, zone, interface):
         return dbus_to_python(self.fw_zone.removeInterface(zone, interface))
 
+    # sources
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addSource(self, zone, source):
+        return dbus_to_python(self.fw_zone.addSource(zone, source))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def changeZoneOfSource(self, zone, source):
+        return dbus_to_python(self.fw_zone.changeZoneOfSource(zone, source))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getSources(self, zone):
+        return dbus_to_python(self.fw_zone.getSources(zone))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def querySource(self, zone, source):
+        return dbus_to_python(self.fw_zone.querySource(zone, source))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeSource(self, zone, source):
+        return dbus_to_python(self.fw_zone.removeSource(zone, source))
+
+    # rich rules
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addRichRule(self, zone, rule, timeout=0):
+        return dbus_to_python(self.fw_zone.addRichRule(zone, rule, timeout))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getRichRules(self, zone):
+        return dbus_to_python(self.fw_zone.getRichRules(zone))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def queryRichRule(self, zone, rule):
+        return dbus_to_python(self.fw_zone.queryRichRule(zone, rule))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeRichRule(self, zone, rule):
+        return dbus_to_python(self.fw_zone.removeRichRule(zone, rule))
+
     # services
 
     @slip.dbus.polkit.enable_proxy
@@ -1042,13 +1352,13 @@ class FirewallClient(object):
 
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
-    def removeRule(self, ipv, table, chain, args):
-        self.fw_direct.removeRule(ipv, table, chain, args)
+    def removeRule(self, ipv, table, chain, priority, args):
+        self.fw_direct.removeRule(ipv, table, chain, priority, args)
 
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
-    def queryRule(self, ipv, table, chain, args):
-        return dbus_to_python(self.fw_direct.queryRule(ipv, table, chain, args))
+    def queryRule(self, ipv, table, chain, priority, args):
+        return dbus_to_python(self.fw_direct.queryRule(ipv, table, chain, priority, args))
 
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
@@ -1060,4 +1370,112 @@ class FirewallClient(object):
     @slip.dbus.polkit.enable_proxy
     @handle_exceptions
     def passthrough(self, ipv, args):
-        return self.fw_direct.passthrough(ipv, args)
+        return dbus_to_python(self.fw_direct.passthrough(ipv, args))
+
+    # lockdown
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def enableLockdown(self):
+        self.fw_policies.enableLockdown()
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def disableLockdown(self):
+        self.fw_policies.disableLockdown()
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def queryLockdown(self):
+        return dbus_to_python(self.fw_policies.queryLockdown())
+
+    # policies
+
+    # lockdown white list commands
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addLockdownWhitelistCommand(self, command):
+        self.fw_policies.addLockdownWhitelistCommand(command)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getLockdownWhitelistCommands(self):
+        return dbus_to_python(self.fw_policies.getLockdownWhitelistCommands())
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def queryLockdownWhitelistCommand(self, command):
+        return dbus_to_python(self.fw_policies.queryLockdownWhitelistCommand(command))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeLockdownWhitelistCommand(self, command):
+        self.fw_policies.removeLockdownWhitelistCommand(command)
+
+    # lockdown white list contexts
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addLockdownWhitelistContext(self, context):
+        self.fw_policies.addLockdownWhitelistContext(context)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getLockdownWhitelistContexts(self):
+        return dbus_to_python(self.fw_policies.getLockdownWhitelistContexts())
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def queryLockdownWhitelistContext(self, context):
+        return dbus_to_python(self.fw_policies.queryLockdownWhitelistContext(context))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeLockdownWhitelistContext(self, context):
+        self.fw_policies.removeLockdownWhitelistContext(context)
+
+    # lockdown white list uids
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addLockdownWhitelistUid(self, uid):
+        self.fw_policies.addLockdownWhitelistUid(uid)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getLockdownWhitelistUids(self):
+        return dbus_to_python(self.fw_policies.getLockdownWhitelistUids())
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def queryLockdownWhitelistUid(self, uid):
+        return dbus_to_python(self.fw_policies.queryLockdownWhitelistUid(uid))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeLockdownWhitelistUid(self, uid):
+        self.fw_policies.removeLockdownWhitelistUid(uid)
+
+    # lockdown white list users
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def addLockdownWhitelistUser(self, user):
+        self.fw_policies.addLockdownWhitelistUser(user)
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def getLockdownWhitelistUsers(self):
+        return dbus_to_python(self.fw_policies.getLockdownWhitelistUsers())
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def queryLockdownWhitelistUser(self, user):
+        return dbus_to_python(self.fw_policies.queryLockdownWhitelistUser(user))
+
+    @slip.dbus.polkit.enable_proxy
+    @handle_exceptions
+    def removeLockdownWhitelistUser(self, user):
+        self.fw_policies.removeLockdownWhitelistUser(user)
+
