@@ -29,9 +29,7 @@ except locale.Error:
 
 DOMAIN = 'firewalld'
 import gettext
-_ = lambda x: gettext.ldgettext(DOMAIN, x)
-import __builtin__
-__builtin__.__dict__['_'] = _
+gettext.install(domain=DOMAIN)
 
 # configuration
 DAEMON_NAME = 'firewalld'
@@ -40,7 +38,7 @@ APPLET_NAME = 'firewall-applet'
 DATADIR = '/usr/share/' + DAEMON_NAME
 CONFIG_GLADE_NAME = CONFIG_NAME + '.glade'
 COPYRIGHT = '(C) 2010-2012 Red Hat, Inc.'
-VERSION = '0.3.0'
+VERSION = '0.3.4'
 AUTHORS = [
     "Thomas Woerner <twoerner@redhat.com>",
     "Jiri Popelka <jpopelka@redhat.com>",
@@ -73,6 +71,8 @@ FIREWALLD_ICMPTYPES = USR_LIB_FIREWALLD + '/icmptypes'
 
 FIREWALLD_LOGFILE = '/var/log/firewalld'
 
+FIREWALLD_DIRECT = ETC_FIREWALLD + '/direct.xml'
+
 LOCKDOWN_WHITELIST = ETC_FIREWALLD + '/lockdown-whitelist.xml'
 
 SYSCTL_CONFIG = '/etc/sysctl.conf'
