@@ -45,8 +45,8 @@ class ebtables:
         log.debug2("%s: %s %s", self.__class__, self._command, " ".join(_args))
         (status, ret) = runProg(self._command, _args)
         if status != 0:
-            raise ValueError, "'%s %s' failed: %s" % (self._command, 
-                                                      " ".join(args), ret)
+            raise ValueError("'%s %s' failed: %s" % (self._command,
+                                                     " ".join(args), ret))
         return ret
 
     def set_rule(self, rule):
@@ -59,7 +59,7 @@ class ebtables:
         self.__run([ "-D" ] + rule)
 
     def used_tables(self):
-        return CHAINS.keys()
+        return list(CHAINS.keys())
 
     def flush(self):
         tables = self.used_tables()
@@ -74,7 +74,7 @@ class ebtables:
         if which == "used":
             tables = self.used_tables()
         else:
-            tables = CHAINS.keys()
+            tables = list(CHAINS.keys())
 
         if "nat" in tables:
             tables.remove("nat") # nat can not set policies in nat table
