@@ -31,12 +31,13 @@ from firewall.core.fw_service import FirewallService
 from firewall.core.fw_zone import FirewallZone
 from firewall.core.fw_direct import FirewallDirect
 from firewall.core.fw_config import FirewallConfig
+from firewall.core.fw_policies import FirewallPolicies
 from firewall.core.logger import log
 from firewall.core.io.firewalld_conf import firewalld_conf
-from firewall.core.io.lockdown_whitelist import LockdownWhitelist
+from firewall.core.io.direct import Direct
 from firewall.core.io.service import service_reader
 from firewall.core.io.icmptype import icmptype_reader
-from firewall.core.io.zone import zone_reader
+from firewall.core.io.zone import zone_reader, Zone
 from firewall.errors import *
 
 ############################################################################
@@ -63,7 +64,7 @@ class Firewall:
         self.zone = FirewallZone(self)
         self.direct = FirewallDirect(self)
         self.config = FirewallConfig(self)
-        self.lockdown_whitelist = LockdownWhitelist(LOCKDOWN_WHITELIST)
+        self.policies = FirewallPolicies(self)
 
         self.__init_vars()
 
@@ -75,7 +76,6 @@ class Firewall:
         self._marks = [ ]
         self._min_mark = FALLBACK_MINIMAL_MARK # will be overloaded by firewalld.conf
         self.cleanup_on_exit = True
-        self.lockdown = False
 
     def start(self):
         # check if iptables, ip6tables and ebtables are usable, else disable
@@ -118,50 +118,59 @@ class Firewall:
                 default_zone = self._firewalld_conf.get("DefaultZone")
             if self._firewalld_conf.get("MinimalMark"):
                 mark = self._firewalld_conf.get("MinimalMark")
-                try:
-                    self._min_mark = int(mark)
-                except Exception as msg:
-                    log.error("MinimalMark %s is not valid, using default "
-                              "value %d", mark, self._min_mark)
+                if mark != None:
+                    try:
+                        self._min_mark = int(mark)
+                    except Exception as msg:
+                        log.error("MinimalMark %s is not valid, using default "
+                                  "value %d", mark, self._min_mark)
             if self._firewalld_conf.get("CleanupOnExit"):
                 value = self._firewalld_conf.get("CleanupOnExit")
-                if value.lower() in [ "no", "false" ]:
+                if value != None and value.lower() in [ "no", "false" ]:
                     self.cleanup_on_exit = False
 
             if self._firewalld_conf.get("Lockdown"):
                 value = self._firewalld_conf.get("Lockdown")
-                if value.lower() in [ "yes", "true" ]:
+                if value != None and value.lower() in [ "yes", "true" ]:
                     log.debug1("Lockdown is enabled")
-                    self.lockdown = True
+                    try:
+                        self.policies.enable_lockdown()
+                    except FirewallError:
+                        # already enabled, this is probably reload
+                        pass
+
+        self.config.set_firewalld_conf(copy.deepcopy(self._firewalld_conf))
 
         # apply default rules
         self._apply_default_rules()
 
         # load lockdown whitelist
-        if self.lockdown:
-            log.debug1("Loading lockdown whitelist")
-            try:
-                self.lockdown_whitelist.read()
-            except Exception as msg:
-                log.error("Failed to load lockdown whitelist '%s': %s",
-                          LOCKDOWN_WHITELIST, msg)
+        log.debug1("Loading lockdown whitelist")
+        try:
+            self.policies.lockdown_whitelist.read()
+        except Exception as msg:
+            log.error("Failed to load lockdown whitelist '%s': %s",
+                      self.policies.lockdown_whitelist.filename, msg)
+
+        # copy policies to config interface
+        self.config.set_policies(copy.deepcopy(self.policies))
 
         # load icmptype files
-        self._loader(FIREWALLD_ICMPTYPES, "icmptype", True)
+        self._loader(FIREWALLD_ICMPTYPES, "icmptype")
         self._loader(ETC_FIREWALLD_ICMPTYPES, "icmptype")
 
         if len(self.icmptype.get_icmptypes()) == 0:
             log.error("No icmptypes found.")
 
         # load service files
-        self._loader(FIREWALLD_SERVICES, "service", True)
+        self._loader(FIREWALLD_SERVICES, "service")
         self._loader(ETC_FIREWALLD_SERVICES, "service")
 
         if len(self.service.get_services()) == 0:
             log.error("No services found.")
 
         # load zone files
-        self._loader(FIREWALLD_ZONES, "zone", True)
+        self._loader(FIREWALLD_ZONES, "zone")
         self._loader(ETC_FIREWALLD_ZONES, "zone")
 
         if len(self.zone.get_zones()) == 0:
@@ -177,6 +186,22 @@ class Firewall:
         if error:
             sys.exit(1)
 
+        # load direct rules
+        log.debug1("Loading direct rules file '%s'" % FIREWALLD_DIRECT)
+        obj = Direct(FIREWALLD_DIRECT)
+        try:
+            obj.read()
+        except Exception as msg:
+            log.debug1("Failed to load direct rules file '%s': %s",
+                      FIREWALLD_DIRECT, msg)
+        else:
+            #obj.output()
+            self.direct.set_config((obj.get_all_chains(), obj.get_all_rules()))
+            for ipv, args in obj.get_all_passthroughs().items():
+                for arg in args:
+                    self.direct.passthrough(ipv, arg)
+            # TODO: copy obj into config interface
+
         # check if default_zone is a valid zone
         if default_zone not in self.zone.get_zones():
             if "public" in self.zone.get_zones():
@@ -197,12 +222,28 @@ class Firewall:
 
         self._state = "RUNNING"
 
-    def _loader(self, path, reader_type, default=False):
+    def _loader(self, path, reader_type, combine=False):
+        # combine: several zone files are getting combined into one obj
         if not os.path.isdir(path):
             return
 
-        for filename in os.listdir(path):
+        if combine == True:
+            if path.startswith(ETC_FIREWALLD) and reader_type == "zone":
+                combined_zone = Zone()
+                combined_zone.name = os.path.basename(path)
+                combined_zone.check_name(combined_zone.name)
+                combined_zone.path = path
+                combined_zone.default = False
+            else:
+                combine = False
+
+        for filename in sorted(os.listdir(path)):
             if not filename.endswith(".xml"):
+                if path.startswith(ETC_FIREWALLD) and \
+                        reader_type == "zone" and \
+                        os.path.isdir("%s/%s" % (path, filename)):
+                    self._loader("%s/%s" % (path, filename), reader_type,
+                                 combine=True)
                 continue
 
             name = "%s/%s" % (path, filename)
@@ -218,7 +259,7 @@ class Firewall:
                         self.icmptype.remove_icmptype(orig_obj.name)
                     self.icmptype.add_icmptype(obj)
                     # add a deep copy to the configuration interface
-                    self.config.add_icmptype(copy.deepcopy(obj), default)
+                    self.config.add_icmptype(copy.deepcopy(obj))
                 elif reader_type == "service":
                     obj = service_reader(filename, path)
                     if obj.name in self.service.get_services():
@@ -229,20 +270,26 @@ class Firewall:
                         self.service.remove_service(orig_obj.name)
                     self.service.add_service(obj)
                     # add a deep copy to the configuration interface
-                    self.config.add_service(copy.deepcopy(obj), default)
+                    self.config.add_service(copy.deepcopy(obj))
                 elif reader_type == "zone":
                     obj = zone_reader(filename, path)
-                    if obj.name in self.zone.get_zones():
-                        orig_obj = self.zone.get_zone(obj.name)
-                        if orig_obj.immutable:
-                            raise FirewallError(NOT_OVERLOADABLE, obj.name)
-                        log.debug1("  Overloads %s '%s' ('%s/%s')", reader_type,
-                                   orig_obj.name, orig_obj.path,
-                                   orig_obj.filename)
-                        self.zone.remove_zone(orig_obj.name)
-                    self.zone.add_zone(obj)
-                    # add a deep copy to the configuration interface
-                    self.config.add_zone(copy.deepcopy(obj), default)
+                    if not combine:
+                        if obj.name in self.zone.get_zones():
+                            orig_obj = self.zone.get_zone(obj.name)
+                            if orig_obj.combined:
+                                raise FirewallError(NOT_OVERLOADABLE,
+                                                    "%s is a combined zone" % \
+                                                        obj.name)
+                            log.debug1("  Overloads %s '%s' ('%s/%s')",
+                                       reader_type,
+                                       orig_obj.name, orig_obj.path,
+                                       orig_obj.filename)
+                            self.zone.remove_zone(orig_obj.name)
+                        self.zone.add_zone(obj)
+                        # add a deep copy to the configuration interface
+                        self.config.add_zone(copy.deepcopy(obj))
+                    else:
+                        combined_zone.combine(obj)
                 else:
                     log.fatal("Unknown reader type %s", reader_type)
             except FirewallError as msg:
@@ -252,6 +299,19 @@ class Firewall:
                 log.error("Failed to load %s file '%s':", reader_type, name)
                 log.exception()
 
+        if combine == True and combined_zone.combined == True:
+            if combined_zone.name in self.zone.get_zones():
+                orig_obj = self.zone.get_zone(combined_zone.name)
+                log.debug1("  Overloading and deactivating %s '%s' ('%s/%s')",
+                           reader_type, orig_obj.name, orig_obj.path,
+                           orig_obj.filename)
+                try:
+                    self.zone.remove_zone(combined_zone.name)
+                except:
+                    pass
+                self.config.forget_zone(combined_zone.name)
+            self.zone.add_zone(combined_zone)
+
     def cleanup(self):
         self.__init_vars()
         self.icmptype.cleanup()
@@ -268,9 +328,6 @@ class Firewall:
 
         self.cleanup()
 
-    def lockdown_enabled(self):
-        return self.lockdown
-
     # marks
 
     def new_mark(self):
@@ -511,31 +568,6 @@ class Firewall:
     def check_icmptype(self, icmp):
         self.icmptype.check_icmptype(icmp)
 
-    # access check
-
-    def access_check(self, key, value):
-        if key == "context":
-            log.debug2('Doing access check for context "%s"' % value)
-            if self.lockdown_whitelist.match_context(value):
-                log.debug3('context matches.')
-                return True
-        elif key == "uid":
-            log.debug2('Doing access check for uid %d' % value)
-            if self.lockdown_whitelist.match_uid(value):
-                log.debug3('uid matches.')
-                return True
-        elif key == "user":
-            log.debug2('Doing access check for user "%s"' % value)
-            if self.lockdown_whitelist.match_user(value):
-                log.debug3('user matches.')
-                return True
-        elif key == "command":
-            log.debug2('Doing access check for command "%s"' % value)
-            if self.lockdown_whitelist.match_command(value):
-                log.debug3('command matches.')
-                return True
-        return False
-
     # RELOAD
 
     def reload(self, stop=False):
@@ -585,7 +617,7 @@ class Firewall:
             else:
                 log.info1("New zone '%s'.", zone)
         if len(_zone_interfaces) > 0:
-            for zone in _zone_interfaces:
+            for zone in _zone_interfaces.keys():
                 log.info1("Lost zone '%s', zone interfaces dropped.", zone)
                 del _zone_interfaces[zone]
         del _zone_interfaces
