@@ -42,6 +42,8 @@ class FirewallConfig:
         self._default_icmptypes = { }
         self._default_services = { }
         self._default_zones = { }
+        self._firewalld_conf = None
+        self._policies = None
 
     def cleanup(self):
         for x in self._default_icmptypes.keys():
@@ -64,19 +66,35 @@ class FirewallConfig:
     # access check
 
     def lockdown_enabled(self):
-        return self._fw.lockdown_enabled()
+        return self._fw.policies.query_lockdown()
 
     def access_check(self, key, value):
-        return self._fw.access_check(key, value)
+        return self._fw.policies.access_check(key, value)
+
+    # firewalld_conf
+
+    def set_firewalld_conf(self, conf):
+        self._firewalld_conf = conf
+
+    def get_firewalld_conf(self):
+        return self._firewalld_conf
+
+    # policies
+
+    def set_policies(self, policies):
+        self._policies = policies
+
+    def get_policies(self):
+        return self._policies
 
     # icmptypes
 
     def get_icmptypes(self):
-        return list(set(self._icmptypes.keys() + self._default_icmptypes.keys()))
+        return list(set(list(self._icmptypes.keys()) + \
+                        list(self._default_icmptypes.keys())))
 
-    def add_icmptype(self, obj, default):
-        obj.defaults = default
-        if default:
+    def add_icmptype(self, obj):
+        if obj.default:
             self._default_icmptypes[obj.name] = obj
         else:
             self._icmptypes[obj.name] = obj
@@ -105,11 +123,12 @@ class FirewallConfig:
         return obj.export_config()
 
     def set_icmptype_config(self, obj, config):
-        if obj.defaults:
+        if obj.default:
             x = copy.copy(obj)
             x.import_config(config)
             x.path = ETC_FIREWALLD_ICMPTYPES
-            self.add_icmptype(x, False)
+            x.default = False
+            self.add_icmptype(x)
             icmptype_writer(x)
             return x
         else:
@@ -131,9 +150,10 @@ class FirewallConfig:
         x.name = name
         x.filename = "%s.xml" % name
         x.path = ETC_FIREWALLD_ICMPTYPES
+        x.default = False
 
         icmptype_writer(x)
-        self.add_icmptype(x, False)
+        self.add_icmptype(x)
         return x
 
     def update_icmptype_from_path(self, name):
@@ -171,14 +191,10 @@ class FirewallConfig:
         # new or updated file
 
         obj = icmptype_reader(filename, path)
-        obj.defaults = False
 
         # new icmptype
         if obj.name not in self._default_icmptypes and obj.name not in self._icmptypes:
-            if path == ETC_FIREWALLD_ICMPTYPES:
-                self.add_icmptype(obj, False)
-            else:
-                self.add_icmptype(obj, True)
+            self.add_icmptype(obj)
             return ("new", obj)
 
         # updated icmptype
@@ -212,7 +228,7 @@ class FirewallConfig:
         del self._icmptypes[obj.name]
 
     def is_builtin_icmptype(self, obj):
-        if obj.defaults or obj.name in self._default_icmptypes:
+        if obj.default or obj.name in self._default_icmptypes:
             return True
         return False
 
@@ -236,11 +252,11 @@ class FirewallConfig:
     # services
 
     def get_services(self):
-        return list(set(self._services.keys() + self._default_services.keys()))
+        return list(set(list(self._services.keys()) + \
+                        list(self._default_services.keys())))
 
-    def add_service(self, obj, default):
-        obj.defaults = default
-        if default:
+    def add_service(self, obj):
+        if obj.default:
             self._default_services[obj.name] = obj
         else:
             self._services[obj.name] = obj
@@ -269,11 +285,12 @@ class FirewallConfig:
         return obj.export_config()
 
     def set_service_config(self, obj, config):
-        if obj.defaults:
+        if obj.default:
             x = copy.copy(obj)
             x.import_config(config)
             x.path = ETC_FIREWALLD_SERVICES
-            self.add_service(x, False)
+            x.default = False
+            self.add_service(x)
             service_writer(x)
             return x
         else:
@@ -295,9 +312,10 @@ class FirewallConfig:
         x.name = name
         x.filename = "%s.xml" % name
         x.path = ETC_FIREWALLD_SERVICES
+        x.default = False
 
         service_writer(x)
-        self.add_service(x, False)
+        self.add_service(x)
         return x
 
     def update_service_from_path(self, name):
@@ -335,14 +353,10 @@ class FirewallConfig:
         # new or updated file
 
         obj = service_reader(filename, path)
-        obj.defaults = False
 
         # new service
         if obj.name not in self._default_services and obj.name not in self._services:
-            if path == ETC_FIREWALLD_SERVICES:
-                self.add_service(obj, False)
-            else:
-                self.add_service(obj, True)
+            self.add_service(obj)
             return ("new", obj)
 
         # updated service
@@ -376,7 +390,7 @@ class FirewallConfig:
         del self._services[obj.name]
 
     def is_builtin_service(self, obj):
-        if obj.defaults or obj.name in self._default_services:
+        if obj.default or obj.name in self._default_services:
             return True
         return False
 
@@ -400,15 +414,21 @@ class FirewallConfig:
     # zones
 
     def get_zones(self):
-        return list(set(self._zones.keys() + self._default_zones.keys()))
+        return list(set(self._zones.keys() + \
+                        self._default_zones.keys()))
 
-    def add_zone(self, obj, default):
-        obj.defaults = default
-        if default:
+    def add_zone(self, obj):
+        if obj.default:
             self._default_zones[obj.name] = obj
         else:
             self._zones[obj.name] = obj
 
+    def forget_zone(self, name):
+        if name in self._default_zones:
+            del self._default_zones[name]
+        if name in self._zones:
+            del self._zones[name]
+
     def get_zone(self, name):
         if name in self._zones:
             return self._zones[name]
@@ -433,12 +453,13 @@ class FirewallConfig:
         return obj.export_config()
 
     def set_zone_config(self, obj, config):
-        if obj.defaults:
+        if obj.default:
             x = copy.copy(obj)
             x.fw_config = self
             x.import_config(config)
             x.path = ETC_FIREWALLD_ZONES
-            self.add_zone(x, False)
+            x.default = False
+            self.add_zone(x)
             zone_writer(x)
             return x
         else:
@@ -462,9 +483,10 @@ class FirewallConfig:
         x.name = name
         x.filename = "%s.xml" % name
         x.path = ETC_FIREWALLD_ZONES
+        x.default = False
 
         zone_writer(x)
-        self.add_zone(x, False)
+        self.add_zone(x)
         return x
 
     def update_zone_from_path(self, name):
@@ -503,14 +525,10 @@ class FirewallConfig:
 
         obj = zone_reader(filename, path)
         obj.fw_config = self
-        obj.defaults = False
 
         # new zone
         if obj.name not in self._default_zones and obj.name not in self._zones:
-            if path == ETC_FIREWALLD_ZONES:
-                self.add_zone(obj, False)
-            else:
-                self.add_zone(obj, True)
+            self.add_zone(obj)
             return ("new", obj)
 
         # updated zone
@@ -544,7 +562,7 @@ class FirewallConfig:
         del self._zones[obj.name]
 
     def is_builtin_zone(self, obj):
-        if obj.defaults or obj.name in self._default_zones:
+        if obj.default or obj.name in self._default_zones:
             return True
         return False
 
