@@ -22,6 +22,7 @@
 import os.path
 from firewall.config import *
 from firewall import functions
+from firewall.fw_types import *
 from firewall.core import ipXtables
 from firewall.core import ebtables
 from firewall.core import modules
@@ -47,8 +48,8 @@ class FirewallDirect:
         self.__init_vars()
 
     def __init_vars(self):
-        self._chains = { }
-        self._rules = { }
+        self._chains = LastUpdatedOrderedDict()
+        self._rules = LastUpdatedOrderedDict()
         self._rule_priority_positions = { }
 
     def cleanup(self):
@@ -62,12 +63,14 @@ class FirewallDirect:
         for table_id in _chains:
             (ipv, table) = table_id
             for chain in _chains[table_id]:
-                self.__chain(True, ipv, table, chain)
+                if not self.query_chain(ipv, table, chain):
+                    self.add_chain(ipv, table, chain)
+
         for chain_id in _rules:
             (ipv, table, chain) = chain_id
-            for args in _rules[chain_id]:
-                priority = _rules[chain_id][args]
-                self.__rule(True, ipv, table, chain, priority, args)
+            for (priority, args) in _rules[chain_id]:
+                if not self.query_rule(ipv, table, chain, priority, args):
+                    self.add_rule(ipv, table, chain, priority, args)
 
     # DIRECT CHAIN
 
@@ -137,17 +140,18 @@ class FirewallDirect:
             _chain = "%s_direct" % (chain)
 
         chain_id = (ipv, table, _chain)
+        rule_id = (priority, args)
 
         if enable:
             if chain_id in self._rules and \
-                    args in self._rules[chain_id]:
+                    rule_id in self._rules[chain_id]:
                 raise FirewallError(ALREADY_ENABLED)
         else:
             if not chain_id in self._rules or \
-                    not args in self._rules[chain_id]:
+                    not rule_id in self._rules[chain_id]:
                 raise FirewallError(NOT_ENABLED)
             # get priority of rule
-            priority = self._rules[chain_id][args]
+            priority = self._rules[chain_id][rule_id]
 
         # sum number of rules for all positions up to position (including)
         # sort used positions
@@ -161,7 +165,7 @@ class FirewallDirect:
 
         rule = [ "-t", table ]
         if enable:
-            rule += [ "-I", _chain, index ]
+            rule += [ "-I", _chain, str(index) ]
         else:
             rule += [ "-D", _chain ]
         rule += args
@@ -174,8 +178,8 @@ class FirewallDirect:
 
         if enable:
             if not chain_id in self._rules:
-                self._rules[chain_id] = { }
-            self._rules[chain_id][args] = priority
+                self._rules[chain_id] = LastUpdatedOrderedDict()
+            self._rules[chain_id][rule_id] = priority
             if chain_id not in self._rule_priority_positions:
                 self._rule_priority_positions[chain_id] = { }
 
@@ -184,7 +188,7 @@ class FirewallDirect:
             else:
                 self._rule_priority_positions[chain_id][priority] = 1
         else:
-            del self._rules[chain_id][args]
+            del self._rules[chain_id][rule_id]
             if len(self._rules[chain_id]) == 0:
                 del self._rules[chain_id]
             self._rule_priority_positions[chain_id][priority] -= 1
@@ -192,11 +196,10 @@ class FirewallDirect:
     def add_rule(self, ipv, table, chain, priority, args):
         self.__rule(True, ipv, table, chain, priority, args)
 
-    def remove_rule(self, ipv, table, chain, args):
-        # priority of rule will be gathered in __rule
-        self.__rule(False, ipv, table, chain, 0, args)
+    def remove_rule(self, ipv, table, chain, priority, args):
+        self.__rule(False, ipv, table, chain, priority, args)
 
-    def query_rule(self, ipv, table, chain, args):
+    def query_rule(self, ipv, table, chain, priority, args):
         _chain = chain
         # use "%s_chain" for built-in chains
         if ipv in [ "ipv4", "ipv6" ]:
@@ -207,7 +210,7 @@ class FirewallDirect:
             _chain = "%s_direct" % (chain)
         chain_id = (ipv, table, _chain)
         return (chain_id in self._rules and \
-                    args in self._rules[chain_id])
+                (priority, args) in self._rules[chain_id])
 
     def get_rules(self, ipv, table, chain):
         _chain = chain
