@@ -23,7 +23,7 @@ import time
 from firewall.core.base import *
 from firewall.core.logger import log
 from firewall.functions import portStr, checkIPnMask, checkIP6nMask, \
-    checkProtocol
+    checkProtocol, enable_ip_forwarding, check_single_address
 from firewall.core.rich import *
 from firewall.errors import *
 
@@ -42,7 +42,7 @@ class FirewallZone:
     # zones
 
     def get_zones(self):
-        return self._zones.keys()
+        return list(self._zones.keys())
 
     def get_zone_of_interface(self, interface):
         interface_id = self.__interface_id(interface)
@@ -52,13 +52,13 @@ class FirewallZone:
                 return zone
         return None
 
-    def get_zone_of_source(self, ipv, source):
-        source_id = self.__source_id(ipv, source)
+    def get_zone_of_source(self, source):
+        source_id = self.__source_id(source)
         for zone in self._zones:
             if source_id in self._zones[zone].settings["sources"]:
                 # a source_id can only be part of one zone
                 return zone
-        return None        
+        return None
 
     def get_zone(self, zone):
         z = self._fw.check_zone(zone)
@@ -68,7 +68,9 @@ class FirewallZone:
         # transform errors into warnings
         try:
             f(name, *args)
-        except FirewallError as msg:
+        except FirewallError as error:
+            msg = str(error)
+            code = FirewallError.get_code(msg)
             log.warning("%s: %s" % (name, msg))
 
     def add_zone(self, obj):
@@ -83,7 +85,7 @@ class FirewallZone:
         for args in obj.interfaces:
             self._error2warning(self.add_interface, obj.name, args)
         for args in obj.sources:
-            self._error2warning(self.add_source, obj.name, *args)
+            self._error2warning(self.add_source, obj.name, args)
         for args in obj.icmp_blocks:
             self._error2warning(self.add_icmp_block, obj.name, args)
         for args in obj.forward_ports:
@@ -102,7 +104,7 @@ class FirewallZone:
         for args in obj.interfaces:
             self._error2warning(self.remove_interface, obj.name, args)
         for args in obj.sources:
-            self._error2warning(self.remove_source, obj.name, *args)
+            self._error2warning(self.remove_source, obj.name, args)
         for args in obj.icmp_blocks:
             self._error2warning(self.remove_icmp_block, obj.name, args)
         for args in obj.forward_ports:
@@ -119,14 +121,6 @@ class FirewallZone:
         obj.settings.clear()
         del self._zones[zone]
 
-    def is_immutable(self, zone):
-        z = self._fw.check_zone(zone)
-        return self._zones[z].immutable
-
-    def check_immutable(self, zone):
-        if self.is_immutable(zone):
-            raise FirewallError(IMMUTABLE, zone)
-
     # dynamic chain handling
 
     def __chain(self, zone, create, table, chain):
@@ -134,10 +128,6 @@ class FirewallZone:
                 chain in self._chains[zone][table]:
             return
 
-        # do not create chains for immuable zones
-        if self.is_immutable(zone):
-            return
-
         chains = [ ]
         rules = [ ]
         zones = [ DEFAULT_ZONE_TARGET.format(chain=SHORTCUTS[chain],
@@ -283,8 +273,8 @@ class FirewallZone:
             rules_return = self._fw.handle_rules2(rules, enable)
             if rules_return != None:
                 (cleanup_rules, msg) = rules_return
-            cleanup_chains = chains
-            cleanup_modules = modules
+                cleanup_chains = chains
+                cleanup_modules = modules
         else:
             # error loading modules
             (cleanup_modules, msg) = module_return
@@ -310,6 +300,14 @@ class FirewallZone:
             for (table, chain) in chains:
                 self.remove_chain(zone, table, chain)
 
+        # report error case
+        if cleanup_chains != None or cleanup_modules != None or \
+                cleanup_rules != None:
+            log.error(msg)
+            return msg
+
+        return None
+
     # INTERFACES
 
     def check_interface(self, interface):
@@ -423,22 +421,20 @@ class FirewallZone:
         return self.__interface_id(interface) in self.get_settings(zone)["interfaces"]
 
     def get_interfaces(self, zone):
-        return self.get_settings(zone)["interfaces"].keys()
+        return list(self.get_settings(zone)["interfaces"].keys())
 
     # SOURCES
 
-    def check_source(self, ipv, source):
-        if ipv == "ipv4":
-            if not checkIPnMask(source):
-                raise FirewallError(INVALID_ADDR, source)
-        elif ipv == "ipv6":
-            if not checkIP6nMask(source):
-                raise FirewallError(INVALID_ADDR, source)
+    def check_source(self, source):
+        if checkIPnMask(source):
+            return "ipv4"
+        elif checkIP6nMask(source):
+            return "ipv6"
         else:
-            raise FirewallError(INVALID_IPV)
+            raise FirewallError(INVALID_ADDR, source)
 
-    def __source_id(self, ipv, source):
-        self.check_source(ipv, source)
+    def __source_id(self, source):
+        ipv = self.check_source(source)
         return (ipv, source)
 
     def __source(self, enable, zone, ipv, source):
@@ -472,8 +468,9 @@ class FirewallZone:
                     # DROP is not supported in nat table
                     continue
                 # append rule
-                rule = [ "%s_ZONES" % chain, "-t", table,
-                         opt, source, "-j", target ]
+                action = "-g" if "_ZONE_" in target else "-j"
+                rule = [ "%s_ZONES_SOURCE" % chain, "-t", table,
+                         opt, source, action, target ]
                 rules.append((ipv, rule))
 
         # handle rules
@@ -487,19 +484,18 @@ class FirewallZone:
         if not enable:
             self.remove_chain(zone, table, chain)
 
-    def add_source(self, zone, ipv, source, sender=None):
+    def add_source(self, zone, source, sender=None):
         self._fw.check_panic()
         _zone = self._fw.check_zone(zone)
         _obj = self._zones[_zone]
-
-        source_id = self.__source_id(ipv, source)
+        source_id = self.__source_id(source)
 
         if source_id in _obj.settings["sources"]:
             raise FirewallError(ZONE_ALREADY_SET)
-        if self.get_zone_of_source(ipv, source) != None:
+        if self.get_zone_of_source(source) != None:
             raise FirewallError(ZONE_CONFLICT)
 
-        self.__source(True, _zone, ipv, source)
+        self.__source(True, _zone, source_id[0], source_id[1])
 
         _obj.settings["sources"][source_id] = \
             self.__gen_settings(0, sender)
@@ -508,22 +504,22 @@ class FirewallZone:
 
         return _zone
 
-    def change_zone_of_source(self, zone, ipv, source, sender=None):
+    def change_zone_of_source(self, zone, source, sender=None):
         self._fw.check_panic()
-        _old_zone = self.get_zone_of_source(ipv, source)
+        _old_zone = self.get_zone_of_source(source)
         _new_zone = self._fw.check_zone(zone)
 
         if _new_zone == _old_zone:
             return _old_zone
 
         if _old_zone != None:
-            self.remove_source(_old_zone, ipv, interface)
+            self.remove_source(_old_zone, source)
 
-        return self.add_source(zone, ipv, source, sender)
+        return self.add_source(zone, source, sender)
 
-    def remove_source(self, zone, ipv, source):
+    def remove_source(self, zone, source):
         self._fw.check_panic()
-        zos = self.get_zone_of_source(ipv, source)
+        zos = self.get_zone_of_source(source)
         if zos == None:
             raise FirewallError(UNKNOWN_SOURCE, source)
         _zone = zos if zone == "" else self._fw.check_zone(zone)
@@ -531,19 +527,19 @@ class FirewallZone:
             raise FirewallError(ZONE_CONFLICT)
 
         _obj = self._zones[_zone]
-        source_id = self.__source_id(ipv, source)
-        self.__source(False, _zone, ipv, source)
+        source_id = self.__source_id(source)
+        self.__source(False, _zone, source_id[0], source_id[1])
 
         if source_id in _obj.settings["sources"]:
             del _obj.settings["sources"][source_id]
 
         return _zone
 
-    def query_source(self, zone, ipv, source):
-        return self.__source_id(ipv, source) in self.get_settings(zone)["sources"]
+    def query_source(self, zone, source):
+        return self.__source_id(source) in self.get_settings(zone)["sources"]
 
     def get_sources(self, zone):
-        return self.get_settings(zone)["sources"].keys()
+        return [ k[1] for k in self.get_settings(zone)["sources"].keys() ]
 
     # RICH LANGUAGE
 
@@ -564,7 +560,7 @@ class FirewallZone:
         if destination:
             if destination.invert:
                 command.append("!")
-            command += [ "-s", destination.addr ]
+            command += [ "-d", destination.addr ]
 
     def __rule_limit(self, limit):
         if limit:
@@ -589,7 +585,15 @@ class FirewallZone:
             return
         chain = "%s_log" % target
         _command = command[:]
-        _command += [ "-j", "AUDIT", "--type", rule.audit.type ]
+        if type(rule.action) == Rich_Accept:
+            _type = "accept"
+        elif type(rule.action) == Rich_Reject:
+            _type = "reject"
+        elif type(rule.action) ==  Rich_Drop:
+            _type = "drop"
+        else:
+            _type = "unknown"
+        _command += [ "-j", "AUDIT", "--type", _type ]
         _command += self.__rule_limit(rule.audit.limit)
         rules.append((ipv, table, chain, _command))
 
@@ -644,7 +648,9 @@ class FirewallZone:
 
                 table = "filter"
                 chains.append([table, "INPUT" ])
-                modules += svc.modules
+                if type(rule.action) == Rich_Accept:
+                    # only load modules for accept action
+                    modules += svc.modules
                 target = DEFAULT_ZONE_TARGET.format(chain=SHORTCUTS["INPUT"],
                                                     zone=zone)
 
@@ -716,6 +722,9 @@ class FirewallZone:
 
             # MASQUERADE
             elif type(rule.element) == Rich_Masquerade:
+                if enable:
+                    enable_ip_forwarding(ipv)
+
                 chains.append([ "nat", "POSTROUTING" ])
                 chains.append([ "filter", "FORWARD_OUT" ])
 
@@ -740,6 +749,9 @@ class FirewallZone:
 
             # FORWARD PORT
             elif type(rule.element) == Rich_ForwardPort:
+                if enable:
+                    enable_ip_forwarding(ipv)
+
                 port = rule.element.port
                 protocol = rule.element.protocol
                 toport = rule.element.to_port
@@ -847,17 +859,31 @@ class FirewallZone:
                     command += [ "-j", "%%REJECT%%" ]
                     rules.append((ipv, table, "%s_deny" % target, command))
 
+            elif rule.element == None:
+                # source action
+                table = "filter"
+                chains.append([ table, "INPUT" ])
+                target = DEFAULT_ZONE_TARGET.format(chain=SHORTCUTS["INPUT"],
+                                                    zone=zone)
+                command = [ ]
+                self.__rule_source(rule.source, command)
+                self.__rule_log(ipv, table, target, rule, command, rules)
+                self.__rule_audit(ipv, table, target, rule, command, rules)
+                self.__rule_action(ipv, table, target, rule, command, rules)
+
             # EVERYTHING ELSE
             else:
                 raise FirewallError(INVALID_RULE, "Unknown element %s" % 
                                     type(rule.element))
 
-        self.handle_cmr(zone, chains, modules, rules, enable)
+        msg = self.handle_cmr(zone, chains, modules, rules, enable)
+        if msg != None:
+            raise FirewallError(COMMAND_FAILED, msg)
 
+        return mark_id
 
     def add_rule(self, zone, rule, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -865,8 +891,7 @@ class FirewallZone:
         if rule_id in _obj.settings["rules"]:
             raise FirewallError(ALREADY_ENABLED)
 
-        mark = None
-        self.__rule(True, _zone, rule, mark)
+        mark = self.__rule(True, _zone, rule, None)
 
         _obj.settings["rules"][rule_id] = \
             self.__gen_settings(timeout, sender, mark=mark)
@@ -875,7 +900,6 @@ class FirewallZone:
 
     def remove_rule(self, zone, rule):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -898,7 +922,7 @@ class FirewallZone:
         return self.__rule_id(rule) in self.get_settings(zone)["rules"]
 
     def get_rules(self, zone):
-        return self.get_settings(zone)["rules"].keys()
+        return list(self.get_settings(zone)["rules"].keys())
 
     # SERVICES
 
@@ -967,7 +991,6 @@ class FirewallZone:
 
     def add_service(self, zone, service, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -984,7 +1007,6 @@ class FirewallZone:
 
     def remove_service(self, zone, service):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -1003,7 +1025,7 @@ class FirewallZone:
         return (self.__service_id(service) in self.get_settings(zone)["services"])
 
     def get_services(self, zone):
-        return self.get_settings(zone)["services"].keys()
+        return list(self.get_settings(zone)["services"].keys())
 
     # PORTS
 
@@ -1042,7 +1064,6 @@ class FirewallZone:
 
     def add_port(self, zone, port, protocol, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -1059,7 +1080,6 @@ class FirewallZone:
 
     def remove_port(self, zone, port, protocol):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -1078,7 +1098,7 @@ class FirewallZone:
         return self.__port_id(port, protocol) in self.get_settings(zone)["ports"]
 
     def get_ports(self, zone):
-        return self.get_settings(zone)["ports"].keys()
+        return list(self.get_settings(zone)["ports"].keys())
 
     # PROTOCOLS
 
@@ -1095,12 +1115,13 @@ class FirewallZone:
         if enable:
             self.add_chain(zone, "nat", "POSTROUTING")
             self.add_chain(zone, "filter", "FORWARD_OUT")
+            enable_ip_forwarding("ipv4")
 
         rules = [ ]
         for ipv in [ "ipv4" ]: # IPv4 only!
             target = DEFAULT_ZONE_TARGET.format(
                 chain=SHORTCUTS["POSTROUTING"], zone=zone)
-            rules.append((ipv, [ "%s_allow" % (target),
+            rules.append((ipv, [ "%s_allow" % (target), "!", "-i", "lo",
                                  "-t", "nat", "-j", "MASQUERADE" ]))
             # FORWARD_OUT
             target = DEFAULT_ZONE_TARGET.format(
@@ -1121,7 +1142,6 @@ class FirewallZone:
 
     def add_masquerade(self, zone, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -1138,7 +1158,6 @@ class FirewallZone:
 
     def remove_masquerade(self, zone):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -1164,7 +1183,7 @@ class FirewallZone:
         if toport:
             self._fw.check_port(toport)
         if toaddr:
-            self._fw.check_address(ipv, toaddr)
+            check_single_address(ipv, toaddr)
         if not toport and not toaddr:
             raise FirewallError(INVALID_FORWARD)
 
@@ -1196,6 +1215,7 @@ class FirewallZone:
             self.add_chain(zone, "mangle", "PREROUTING")
             self.add_chain(zone, "nat", "PREROUTING")
             self.add_chain(zone, "filter", filter_chain)
+            enable_ip_forwarding("ipv4")
 
         rules = [ ]
         for ipv in [ "ipv4" ]: # IPv4 only!
@@ -1235,7 +1255,6 @@ class FirewallZone:
     def add_forward_port(self, zone, port, protocol, toport=None,
                          toaddr=None, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -1255,7 +1274,6 @@ class FirewallZone:
     def remove_forward_port(self, zone, port, protocol, toport=None,
                             toaddr=None):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -1280,7 +1298,7 @@ class FirewallZone:
         return forward_id in self.get_settings(zone)["forward_ports"]
 
     def get_forward_ports(self, zone):
-        return self.get_settings(zone)["forward_ports"].keys()
+        return list(self.get_settings(zone)["forward_ports"].keys())
 
     # ICMP BLOCK
 
@@ -1334,7 +1352,6 @@ class FirewallZone:
 
     def add_icmp_block(self, zone, icmp, timeout=0, sender=None):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -1351,7 +1368,6 @@ class FirewallZone:
 
     def remove_icmp_block(self, zone, icmp):
         _zone = self._fw.check_zone(zone)
-        self.check_immutable(_zone)
         self._fw.check_panic()
         _obj = self._zones[_zone]
 
@@ -1370,4 +1386,4 @@ class FirewallZone:
         return self.__icmp_block_id(icmp) in self.get_settings(zone)["icmp_blocks"]
 
     def get_icmp_blocks(self, zone):
-        return self.get_settings(zone)["icmp_blocks"].keys()
+        return list(self.get_settings(zone)["icmp_blocks"].keys())
