@@ -22,9 +22,8 @@
 import xml.sax as sax
 import xml.sax.saxutils as saxutils
 import copy
-import types
 
-from firewall.config import _
+from firewall.config import *
 from firewall.errors import *
 from firewall import functions
 
@@ -51,7 +50,7 @@ class IO_Object(object):
     def import_config(self, config):
         self.check_config(config)
         for i,(element,value) in enumerate(self.IMPORT_EXPORT_STRUCTURE):
-            if isinstance(config[i], types.ListType):
+            if isinstance(config[i], list):
                 # remove duplicates
                 setattr(self, element, copy.deepcopy(list(set(config[i]))))
             else:
@@ -86,20 +85,20 @@ class IO_Object(object):
             raise FirewallError(INVALID_TYPE,
                                 "'%s' not of type %s, but %s" % (\
                     config, type(structure), type(config)))
-        if type(structure) == types.ListType:
+        if type(structure) == list:
             # same type elements, else struct
             if len(structure) != 1:
                 raise FirewallError(INVALID_TYPE, "len('%s') != 1" % structure)
             for x in config:
                 self._check_config_structure(x, structure[0])
-        elif type(structure) == types.TupleType:
+        elif type(structure) == tuple:
             if len(structure) != len(config):
                 raise FirewallError(INVALID_TYPE,
                                     "len('%s') != %d" % (config,
                                                          len(structure)))
             for i,value in enumerate(structure):
                 self._check_config_structure(config[i], value)
-        elif type(structure) == types.DictType:
+        elif type(structure) == dict:
             # only one key value pair in structure
             (skey, svalue) = structure.items()[0]
             for (key, value) in config.items():
@@ -168,29 +167,32 @@ class IO_Object_ContentHandler(sax.handler.ContentHandler):
         self.item = item
 
     def startDocument(self):
-        self._element = None
+        self._element = ""
 
     def endElement(self, name):
         if name == "short":
             self.item.short = self._element
-            self._element = None
         elif name == "description":
             self.item.description = self._element
-            self._element = None
 
     def characters(self, content):
-        if self._element != None:
-            self._element += str(content.replace('\n', ' '))
+        self._element = str(content).replace('\n', ' ')
 
 class IO_Object_XMLGenerator(saxutils.XMLGenerator):
     def __init__(self, out):
         saxutils.XMLGenerator.__init__(self, out, "utf-8")
 
     def simpleElement(self, name, attrs):
-        self._write('<' + name)
+        if isinstance(name, bytes):
+            name = name.decode('utf-8')
+        self._write(u'<' + name)
         for (name, value) in attrs.items():
-            self._write(' %s=%s' % (name, saxutils.quoteattr(value)))
-        self._write('/>')
+            if isinstance(name, bytes):
+                name = name.decode('utf-8')
+            if isinstance(value, bytes):
+                value = value.decode('utf-8')
+            self._write(u' %s=%s' % (name, saxutils.quoteattr(value)))
+        self._write(u'/>')
 
 def check_port(port):
     port_range = functions.getPortRange(port)
