@@ -23,10 +23,10 @@ import xml.sax as sax
 import os
 import shutil
 
-from firewall.config import _
 from firewall.errors import *
 from firewall.core.io.io_object import *
 from firewall.core.logger import log
+from firewall.functions import uniqify
 
 class lockdown_whitelist_ContentHandler(IO_Object_ContentHandler):
     def __init__(self, item):
@@ -81,8 +81,12 @@ class LockdownWhitelist(IO_Object):
     """ LockdownWhitelist class """
 
     IMPORT_EXPORT_STRUCTURE = (
+        ( "commands", [ "" ] ),   # as
+        ( "contexts", [ "" ] ),   # as
+        ( "users", [ "" ] ),      # as
+        ( "uids", [ 0 ] )         # ai
         )
-    DBUS_SIGNATURE = '()'
+    DBUS_SIGNATURE = '(asasasai)'
     ADDITIONAL_ALNUM_CHARS = [ "_" ]
     PARSER_REQUIRED_ELEMENT_ATTRS = {
         "whitelist": None,
@@ -93,7 +97,7 @@ class LockdownWhitelist(IO_Object):
         }
     PARSER_OPTIONAL_ELEMENT_ATTRS = {
         "user": [ "id", "name" ],
-        "group": [ "id", "name" ],
+#        "group": [ "id", "name" ],
         }
 
     def __init__(self, filename):
@@ -105,28 +109,30 @@ class LockdownWhitelist(IO_Object):
         pass
 
     def clear(self):
-        self._commands = [ ]
-        self._uids = [ ]
-        self._users = [ ]
-        self._contexts = [ ]
+        self.commands = [ ]
+        self.contexts = [ ]
+        self.users = [ ]
+        self.uids = [ ]
+#        self.groups = [ ]
 
     # commands
 
     def add_command(self, command):
-        if command not in self._commands:
-            self._commands.append(command)
-
-    def has_command(self, command):
-        return (command in self._commands)
+        if command not in self.commands:
+            self.commands.append(command)
 
     def remove_command(self, command):
-        if command in self._commands:
-            self._commands.remove(command)
+        if command in self.commands:
+            self.commands.remove(command)
         else:
-            raise ValueError, 'Command "%s" not in whitelist.' % command
+            raise FirewallError(NOT_ENABLED,
+                                'Command "%s" not in whitelist.' % command)
+
+    def has_command(self, command):
+        return (command in self.commands)
 
     def match_command(self, command):
-        for _command in self._commands:
+        for _command in self.commands:
             if _command.endswith("*"):
                 if command.startswith(_command[:-1]):
                     return True
@@ -135,59 +141,118 @@ class LockdownWhitelist(IO_Object):
                     return True
         return False
 
+    def get_commands(self):
+        return self.commands
+
     # user ids
 
     def add_uid(self, uid):
-        if uid not in self._uids:
-            self._uids.append(uid)
-
-    def has_uid(self, uid):
-        return (uid in self._uids)
+        if uid not in self.uids:
+            self.uids.append(uid)
 
     def remove_uid(self, uid):
-        if uid in self._uids:
-            self._uids.remove(uid)
+        if uid in self.uids:
+            self.uids.remove(uid)
         else:
-            raise ValueError, 'Uid "%s" not in whitelist.' % uid
+            raise FirewallError(NOT_ENABLED,
+                                'Uid "%s" not in whitelist.' % uid)
+
+    def has_uid(self, uid):
+        return (uid in self.uids)
 
     def match_uid(self, uid):
-        return (uid in self._uids)
+        return (uid in self.uids)
+
+    def get_uids(self):
+        return self.uids
 
     # users
 
     def add_user(self, user):
-        if user not in self._users:
-            self._users.append(user)
-
-    def has_user(self, user):
-        return (user in self._users)
+        if user not in self.users:
+            self.users.append(user)
 
     def remove_user(self, user):
-        if user in self._users:
-            self._users.remove(user)
+        if user in self.users:
+            self.users.remove(user)
         else:
-            raise ValueError, 'User "%s" not in whitelist.' % user
+            raise FirewallError(NOT_ENABLED,
+                                'User "%s" not in whitelist.' % user)
+
+    def has_user(self, user):
+        return (user in self.users)
 
     def match_user(self, user):
-        return (user in self._users)
+        return (user in self.users)
+
+    def get_users(self):
+        return self.users
+
+#    # group ids
+#
+#    def add_gid(self, gid):
+#        if gid not in self.gids:
+#            self.gids.append(gid)
+#
+#    def remove_gid(self, gid):
+#        if gid in self.gids:
+#            self.gids.remove(gid)
+#        else:
+#            raise FirewallError(NOT_ENABLED,
+#                                'Gid "%s" not in whitelist.' % gid)
+#
+#    def has_gid(self, gid):
+#        return (gid in self.gids)
+#
+#    def match_gid(self, gid):
+#        return (gid in self.gids)
+#
+#    def get_gids(self):
+#        return self.gids
+
+#    # groups
+#
+#    def add_group(self, group):
+#        if group not in self.groups:
+#            self.groups.append(group)
+#
+#    def remove_group(self, group):
+#        if group in self.groups:
+#            self.groups.remove(group)
+#        else:
+#            raise FirewallError(NOT_ENABLED,
+#                                'Group "%s" not in whitelist.' % group)
+#
+#    def has_group(self, group):
+#        return (group in self.groups)
+#
+#    def match_group(self, group):
+#        return (group in self.groups)
+#
+#    def get_groups(self):
+#        return self.groups
 
     # selinux contexts
 
     def add_context(self, context):
-        if context not in self._contexts:
-            self._contexts.append(context)
-
-    def has_context(self, context):
-        return (context in self._contexts)
+        if context not in self.contexts:
+            self.contexts.append(context)
 
     def remove_context(self, context):
-        if context in self._contexts:
-            self._contexts.remove(context)
+        if context in self.contexts:
+            self.contexts.remove(context)
         else:
-            raise ValueError, 'Context "%s" not in whitelist.' % context
+            raise FirewallError(NOT_ENABLED,
+                                'Context "%s" not in whitelist.' % context)
+
+    def has_context(self, context):
+        return (context in self.contexts)
 
     def match_context(self, context):
-        return (context in self._contexts)
+        return (context in self.contexts)
+
+    def get_contexts(self):
+        return self.contexts
 
     # read and write
 
@@ -205,8 +270,7 @@ class LockdownWhitelist(IO_Object):
             try:
                 shutil.copy2(self.filename, "%s.old" % self.filename)
             except Exception, msg:
-                raise IOError, "Backup of '%s' failed: %s" % (self.filename,
-                                                              msg)
+                raise IOError("Backup of '%s' failed: %s" % (self.filename, msg))
 
         fd = open(self.filename, "w")
         handler = IO_Object_XMLGenerator(fd)
@@ -216,16 +280,39 @@ class LockdownWhitelist(IO_Object):
         handler.startElement("whitelist", { })
         handler.ignorableWhitespace("\n")
 
-        # interfaces
-        for command in set(self._commands):
+        # commands
+        for command in uniqify(self.commands):
             handler.ignorableWhitespace("  ")
             handler.simpleElement("command", { "name": command })
             handler.ignorableWhitespace("\n")
 
-        # TODO: add user, context
+        for uid in uniqify(self.uids):
+            handler.ignorableWhitespace("  ")
+            handler.simpleElement("user", { "id": str(uid) })
+            handler.ignorableWhitespace("\n")
 
-        # end zone element
-        handler.endElement("zone")
+        for user in uniqify(self.users):
+            handler.ignorableWhitespace("  ")
+            handler.simpleElement("user", { "name": user })
+            handler.ignorableWhitespace("\n")
+
+#        for gid in uniqify(self.gids):
+#            handler.ignorableWhitespace("  ")
+#            handler.simpleElement("user", { "id": str(gid) })
+#            handler.ignorableWhitespace("\n")
+
+#        for group in uniqify(self.groups):
+#            handler.ignorableWhitespace("  ")
+#            handler.simpleElement("group", { "name": group })
+#            handler.ignorableWhitespace("\n")
+
+        for context in uniqify(self.contexts):
+            handler.ignorableWhitespace("  ")
+            handler.simpleElement("selinux", { "context": context })
+            handler.ignorableWhitespace("\n")
+
+        # end whitelist element
+        handler.endElement("whitelist")
         handler.ignorableWhitespace("\n")
         handler.endDocument()
         fd.close()
