@@ -23,7 +23,7 @@ import xml.sax as sax
 import os
 import shutil
 
-from firewall.config import _
+from firewall.config import ETC_FIREWALLD
 from firewall.errors import *
 from firewall import functions
 from firewall.core.io.io_object import *
@@ -64,13 +64,21 @@ class Service(IO_Object):
     def _check_config(self, config, item):
         if item == "ports":
             for port in config:
-                check_port(port[0])
-                check_protocol(port[1])
+                if port[0] != "":
+                    check_port(port[0])
+                    check_protocol(port[1])
+                else:
+                    # only protocol
+                    if not functions.checkProtocol(port[1]):
+                        raise FirewallError(INVALID_PROTOCOL, port[1])
+
         elif item == "destination":
             for destination in config:
                 if destination not in [ "ipv4", "ipv6" ]:
                     raise FirewallError(INVALID_DESTINATION, destination)
-                # TODO: check IP address
+                if not functions.check_address(destination, config[destination]):
+                    raise FirewallError(INVALID_ADDRESS, config[destination])
+                    
 
 # PARSER
 
@@ -85,9 +93,9 @@ class service_ContentHandler(IO_Object_ContentHandler):
             if "version" in attrs:
                 self.item.version = str(attrs["version"])
         elif name == "short":
-            self._element = self.item.short
+            pass
         elif name == "description":
-            self._element = self.item.description
+            pass
         elif name == "port":
             self.item.ports.append((str(attrs["port"]),
                                        str(attrs["protocol"])))
@@ -112,17 +120,15 @@ def service_reader(filename, path):
     service.check_name(service.name)
     service.filename = filename
     service.path = path
+    service.default = False if path.startswith(ETC_FIREWALLD) else True
     handler = service_ContentHandler(service)
     parser = sax.make_parser()
     parser.setContentHandler(handler)
     parser.parse(name)
     return service
 
-def service_writer(service, path=""):
-    if path:
-        _path = path
-    else:
-        _path = service.path
+def service_writer(service, path=None):
+    _path = path if path else service.path
 
     if service.filename:
         name = "%s/%s" % (_path, service.filename)
@@ -133,7 +139,7 @@ def service_writer(service, path=""):
         try:
             shutil.copy2(name, "%s.old" % name)
         except Exception, msg:
-            raise IOError, "Backup of '%s' failed: %s" % (name, msg)
+            raise IOError("Backup of '%s' failed: %s" % (name, msg))
 
     fd = open(name, "w")
     handler = IO_Object_XMLGenerator(fd)
@@ -148,6 +154,8 @@ def service_writer(service, path=""):
 
     # short
     if service.short and service.short != "":
+        if isinstance(service.short, bytes):
+            service.short = service.short.decode('utf-8')
         handler.ignorableWhitespace("  ")
         handler.startElement("short", { })
         handler.characters(service.short)
@@ -156,6 +164,8 @@ def service_writer(service, path=""):
 
     # description
     if service.description and service.description != "":
+        if isinstance(service.description, bytes):
+            service.description = service.description.decode('utf-8')
         handler.ignorableWhitespace("  ")
         handler.startElement("description", { })
         handler.characters(service.description)
@@ -182,5 +192,6 @@ def service_writer(service, path=""):
 
     # end service element
     handler.endElement('service')
+    handler.ignorableWhitespace("\n")
     handler.endDocument()
     fd.close()
