@@ -23,9 +23,9 @@ import xml.sax as sax
 import os
 import shutil
 
-from firewall.config import _
+from firewall.config import ETC_FIREWALLD
 from firewall.errors import *
-from firewall import functions
+from firewall.functions import checkIP, uniqify, max_zone_name_len
 from firewall.core.base import DEFAULT_ZONE_TARGET, ZONE_TARGETS
 from firewall.core.io.io_object import *
 from firewall.core.rich import *
@@ -38,15 +38,18 @@ class Zone(IO_Object):
         ( "version",  "" ),                            # s
         ( "short", "" ),                               # s
         ( "description", "" ),                         # s
-        ( "immutable", False ),                        # b
+        ( "UNUSED", False ),                           # b
         ( "target", "" ),                              # s
         ( "services", [ "", ], ),                      # as
         ( "ports", [ ( "", "" ), ], ),                 # a(ss)
         ( "icmp_blocks", [ "", ], ),                   # as
         ( "masquerade", False ),                       # b
         ( "forward_ports", [ ( "", "", "", "" ), ], ), # a(ssss)
+        ( "interfaces", [ "" ] ),                      # as
+        ( "sources", [ "" ] ),                         # as
+        ( "rules_str", [ "" ] ),                       # as
         )
-    DBUS_SIGNATURE = '(sssbsasa(ss)asba(ssss))'
+    DBUS_SIGNATURE = '(sssbsasa(ss)asba(ssss)asasas)'
     ADDITIONAL_ALNUM_CHARS = [ "_" ]
     PARSER_REQUIRED_ELEMENT_ATTRS = {
         "short": None,
@@ -62,7 +65,7 @@ class Zone(IO_Object):
         "destination": [ "address" ],
         "protocol": [ "value" ],
         "log":  None,
-        "audit": [ "type" ],
+        "audit": None,
         "accept": None,
         "reject": None,
         "drop": None,
@@ -73,18 +76,25 @@ class Zone(IO_Object):
         "masquerade": [ "enabled" ],
         "forward-port": [ "to-port", "to-addr" ],
         "rule": [ "family" ],
-        "source": [ "not", "family" ],
-        "destination": [ "not" ],
+        "source": [ "invert", "family" ],
+        "destination": [ "invert" ],
         "log": [ "prefix", "level" ],
         "reject": [ "type" ],
         }
 
+    @staticmethod
+    def index_of (element):
+        for i, (el, val) in enumerate(Zone.IMPORT_EXPORT_STRUCTURE):
+            if el == element:
+                return i
+        raise FirewallError(UNKNOWN_ERROR)
+
     def __init__(self):
         super(Zone, self).__init__()
         self.version = ""
         self.short = ""
         self.description = ""
-        self.immutable = False
+        self.UNUSED = False
         self.target = DEFAULT_ZONE_TARGET
         self.services = [ ]
         self.ports = [ ]
@@ -94,8 +104,21 @@ class Zone(IO_Object):
         self.interfaces = [ ]
         self.sources = [ ]
         self.fw_config = None # to be able to check services and a icmp_blocks
-
         self.rules = [ ]
+        self.combined = False
+
+    def __getattr__(self, name):
+        if name == "rules_str":
+            rules_str = [str(rule) for rule in self.rules]
+            return rules_str
+        else:
+            return object.__getattr__(self, name)
+
+    def __setattr__(self, name, value):
+        if name == "rules_str":
+            self.rules = [Rich_Rule(rule_str=str) for str in value]
+        else:
+            object.__setattr__(self, name, value)
 
     def _check_config(self, config, item):
         if item == "services" and self.fw_config:
@@ -121,9 +144,39 @@ class Zone(IO_Object):
                 if fwd_port[2]:
                     check_port(fwd_port[2])
                 if fwd_port[3]:
-                    if not functions.checkIP(fwd_port[3]):
+                    if not checkIP(fwd_port[3]):
                         raise FirewallError(INVALID_ADDR, fwd_port[3])
 
+    def combine(self, zone):
+        self.combined = True
+        self.filename = None
+        self.version = ""
+        self.short = ""
+        self.description = ""
+
+        for interface in zone.interfaces:
+            if interface not in self.interfaces:
+                self.interfaces.append(interface)
+        for source in zone.sources:
+            if source not in self.sources:
+                self.sources.append(source)
+        for service in zone.services:
+            if service not in self.services:
+                self.services.append(service)
+        for port in zone.ports:
+            if port not in self.ports:
+                self.ports.append(port)
+        for icmp in zone.icmp_blocks:
+            if icmp not in self.icmp_blocks:
+                self.icmp_blocks.append(icmp)
+        if zone.masquerade:
+            self.masquerade = True
+        for forward in zone.forward_ports:
+            if forward not in self.forward_ports:
+                self.forward_ports.append(forward)
+        for rule in zone.rules:
+            self.rules.append(rule)
+
 # PARSER
 
 class zone_ContentHandler(IO_Object_ContentHandler):
@@ -134,7 +187,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
         self._limit_ok = None
 
     def startElement(self, name, attrs):
-        if self._rule != None and self._rule_error:
+        if self._rule_error:
             return
 
         self.item.parser_check_element_attrs(name, attrs)
@@ -145,9 +198,9 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                             attrs["name"])
             if "version" in attrs:
                 self.item.version = str(attrs["version"])
-            if "immutable" in attrs and \
-                    attrs["immutable"].lower() in [ "yes", "true" ]:
-                self.item.immutable = True
+            if "immutable" in attrs:
+                log.warning("Ignoring deprecated attribute immutable='%s'" % 
+                            attrs["immutable"])
             if "target" in attrs:
                 target = str(attrs["target"])
                 if target not in ZONE_TARGETS:
@@ -156,9 +209,9 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                     self.item.target = target
 
         elif name == "short":
-            self._element = self.item.short
+            pass
         elif name == "description":
-            self._element = self.item.description
+            pass
         elif name == "service":
             if self._rule:
                 if self._rule.element:
@@ -204,7 +257,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
         elif name == "masquerade":
             if self._rule:
                 if "enabled" in attrs:
-                    log.warning('Invalid rule: Masquerade attribute ignores in rule.')
+                    log.warning('Invalid rule: Masquerade attribute ignored in rule.')
                 if self._rule.element:
                     log.error('Invalid rule: More than one element, ignoring.')
                     self._rule_error = True
@@ -266,13 +319,13 @@ class zone_ContentHandler(IO_Object_ContentHandler):
             if not "address" in attrs:
                 log.error('Invalid source: Address missing.')
                 return
-            if not "family" in attrs:
-                log.error('Invalid source: Family missing.')
-                return
-            if "not" in attrs:
+            if "family" in attrs:
+                log.warning("Ignoring deprecated attribute family='%s'" %
+                            attrs["family"])
+            if "invert" in attrs:
                 log.error('Invalid source: Invertion not allowed here.')
                 return
-            entry = (str(attrs["family"]), str(attrs["address"]))
+            entry = str(attrs["address"])
             if entry not in self.item.sources:
                 self.item.sources.append(entry)
 
@@ -337,12 +390,7 @@ class zone_ContentHandler(IO_Object_ContentHandler):
                 log.error('Invalid rule: More than one audit')
                 self._rule_error = True
                 return            
-            _type = str(attrs["type"])
-            if _type not in [ "ACCEPT", "REJECT", "DROP" ]:
-                log.error('Invalid rule: Invalid audit type "%s"' % _type)
-                self._rule_error = True
-                return
-            self._rule.audit = Rich_Audit(_type)
+            self._rule.audit = Rich_Audit()
             self._limit_ok = self._rule.audit
 
         elif name == "rule":
@@ -377,9 +425,13 @@ class zone_ContentHandler(IO_Object_ContentHandler):
         IO_Object_ContentHandler.endElement(self, name)
 
         if name == "rule":
-            if not self._rule_error and not self._rule.check():
-                self._rule_error = True
-            if self._rule_error:
+            if not self._rule_error:
+                try:
+                    self._rule.check()
+                except Exception as e:
+                    log.error("%s: %s" % (e, str(self._rule)))
+                    self._rule_error = True
+            if self._rule_error and self._rule in self.item.rules:
                 self.item.rules.remove(self._rule)
             self._rule = None
             self._rule_error = False
@@ -392,9 +444,12 @@ def zone_reader(filename, path):
     if not filename.endswith(".xml"):
         raise FirewallError(INVALID_NAME, filename)
     zone.name = filename[:-4]
+    if len(zone.name) > max_zone_name_len():
+        raise FirewallError(INVALID_NAME, filename)
     zone.check_name(zone.name)
     zone.filename = filename
     zone.path = path
+    zone.default = False if path.startswith(ETC_FIREWALLD) else True
     handler = zone_ContentHandler(zone)
     parser = sax.make_parser()
     parser.setContentHandler(handler)
@@ -402,10 +457,7 @@ def zone_reader(filename, path):
     return zone
 
 def zone_writer(zone, path=None):
-    if path:
-        _path = path
-    else:
-        _path = zone.path
+    _path = path if path else zone.path
 
     if zone.filename:
         name = "%s/%s" % (_path, zone.filename)
@@ -416,7 +468,7 @@ def zone_writer(zone, path=None):
         try:
             shutil.copy2(name, "%s.old" % name)
         except Exception, msg:
-            raise IOError, "Backup of '%s' failed: %s" % (name, msg)
+            raise IOError("Backup of '%s' failed: %s" % (name, msg))
 
     fd = open(name, "w")
     handler = IO_Object_XMLGenerator(fd)
@@ -426,8 +478,6 @@ def zone_writer(zone, path=None):
     attrs = {}
     if zone.version and zone.version != "":
         attrs["version"] = zone.version
-    if zone.immutable:
-        attrs["immutable"] = "yes"
     if zone.target != DEFAULT_ZONE_TARGET:
         attrs["target"] = zone.target
     handler.startElement("zone", attrs)
@@ -435,6 +485,8 @@ def zone_writer(zone, path=None):
 
     # short
     if zone.short and zone.short != "":
+        if isinstance(zone.short, bytes):
+            zone.short = zone.short.decode('utf-8')
         handler.ignorableWhitespace("  ")
         handler.startElement("short", { })
         handler.characters(zone.short)
@@ -443,6 +495,8 @@ def zone_writer(zone, path=None):
 
     # description
     if zone.description and zone.description != "":
+        if isinstance(zone.description, bytes):
+            zone.description = zone.description.decode('utf-8')
         handler.ignorableWhitespace("  ")
         handler.startElement("description", { })
         handler.characters(zone.description)
@@ -450,32 +504,31 @@ def zone_writer(zone, path=None):
         handler.ignorableWhitespace("\n")
 
     # interfaces
-    for interface in set(zone.interfaces):
+    for interface in uniqify(zone.interfaces):
         handler.ignorableWhitespace("  ")
         handler.simpleElement("interface", { "name": interface })
         handler.ignorableWhitespace("\n")
 
     # source
-    for source in set(zone.sources):
+    for source in uniqify(zone.sources):
         handler.ignorableWhitespace("  ")
-        handler.simpleElement("source", { "family": source[0],
-                                          "address": source[1] })
+        handler.simpleElement("source", { "address": source })
         handler.ignorableWhitespace("\n")
 
     # services
-    for service in set(zone.services):
+    for service in uniqify(zone.services):
         handler.ignorableWhitespace("  ")
         handler.simpleElement("service", { "name": service })
         handler.ignorableWhitespace("\n")
 
     # ports
-    for port in set(zone.ports):
+    for port in uniqify(zone.ports):
         handler.ignorableWhitespace("  ")
         handler.simpleElement("port", { "port": port[0], "protocol": port[1] })
         handler.ignorableWhitespace("\n")
 
     # icmp-blocks
-    for icmp in set(zone.icmp_blocks):
+    for icmp in uniqify(zone.icmp_blocks):
         handler.ignorableWhitespace("  ")
         handler.simpleElement("icmp-block", { "name": icmp })
         handler.ignorableWhitespace("\n")
@@ -487,7 +540,7 @@ def zone_writer(zone, path=None):
         handler.ignorableWhitespace("\n")
 
     # forward-ports
-    for forward in set(zone.forward_ports):
+    for forward in uniqify(zone.forward_ports):
         handler.ignorableWhitespace("  ")
         attrs = { "port": forward[0], "protocol": forward[1] }
         if forward[2] and forward[2] != "" :
@@ -583,10 +636,9 @@ def zone_writer(zone, path=None):
 
         # audit
         if rule.audit:
-            attrs = { "type": rule.audit.type }
             if rule.audit.limit:
                 handler.ignorableWhitespace("    ")
-                handler.startElement("audit", attrs)
+                handler.startElement("audit", { })
                 handler.ignorableWhitespace("\n      ")
                 handler.simpleElement("limit",
                                       { "value": rule.audit.limit.value })
