@@ -39,6 +39,7 @@ from firewall.server.config_zone import FirewallDConfigZone
 from firewall.core.io.zone import Zone
 from firewall.core.io.service import Service
 from firewall.core.io.icmptype import IcmpType
+from firewall.core.io.lockdown_whitelist import LockdownWhitelist
 from firewall.dbus_utils import dbus_to_python, \
     command_of_sender, context_of_sender, uid_of_sender, user_of_uid
 from firewall.errors import *
@@ -164,6 +165,15 @@ class FirewallDConfig(slip.dbus.service.Object):
 
     @handle_exceptions
     def removeIcmpType(self, obj):
+        index = 7 # see IMPORT_EXPORT_STRUCTURE in class Zone(IO_Object)
+        for zone in self.zones:
+            settings = zone.getSettings()
+            # if this IcmpType is used in a zone remove it from that zone first
+            if obj.name in settings[index]:
+                settings[index].remove(obj.name)
+                zone.obj = self.config.set_zone_config(zone.obj, settings)
+                zone.Updated(zone.obj.name)
+
         for icmptype in self.icmptypes:
             if icmptype.obj == obj:
                 icmptype.Removed(obj.name)
@@ -193,6 +203,15 @@ class FirewallDConfig(slip.dbus.service.Object):
 
     @handle_exceptions
     def removeService(self, obj):
+        index = 5 # see IMPORT_EXPORT_STRUCTURE in class Zone(IO_Object)
+        for zone in self.zones:
+            settings = zone.getSettings()
+            # if this Service is used in a zone remove it from that zone first
+            if obj.name in settings[index]:
+                settings[index].remove(obj.name)
+                zone.obj = self.config.set_zone_config(zone.obj, settings)
+                zone.Updated(zone.obj.name)
+
         for service in self.services:
             if service.obj == obj:
                 service.Removed(obj.name)
@@ -253,6 +272,139 @@ class FirewallDConfig(slip.dbus.service.Object):
 
     # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 
+    # P R O P E R T I E S
+
+    @dbus_handle_exceptions
+    def _get_property(self, prop):
+        if prop in [ "DefaultZone", "MinimalMark", "CleanupOnExit",
+                     "Lockdown" ]:
+            value = self.config.get_firewalld_conf().get(prop)
+            if prop == "MinimalMark":
+                value = int(value)
+            if value != None:
+                return value
+            if prop == "DefaultZone":
+                return FALLBACK_ZONE
+            elif prop == "MinimalMark":
+                return FALLBACK_MINIMAL_MARK
+            elif prop == "CleanupOnExit":
+                return "yes"
+            elif prop == "Lockdown":
+                return "no"
+        else:
+            raise dbus.exceptions.DBusException(
+                "org.freedesktop.DBus.Error.AccessDenied: "
+                "Property '%s' isn't exported (or may not exist)" % prop)
+
+    @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='ss',
+                         out_signature='v')
+    @dbus_handle_exceptions
+    def Get(self, interface_name, property_name, sender=None):
+        # get a property
+        interface_name = dbus_to_python(interface_name)
+        property_name = dbus_to_python(property_name)
+        log.debug1("config.Get('%s', '%s')", interface_name, property_name)
+
+        if interface_name != DBUS_INTERFACE_CONFIG:
+            raise dbus.exceptions.DBusException(
+                "org.freedesktop.DBus.Error.UnknownInterface: "
+                "FirewallD does not implement %s" % interface_name)
+
+        return self._get_property(property_name)
+
+    @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='s',
+                         out_signature='a{sv}')
+    @dbus_handle_exceptions
+    def GetAll(self, interface_name, sender=None):
+        interface_name = dbus_to_python(interface_name)
+        log.debug1("config.GetAll('%s')", interface_name)
+
+        if interface_name != DBUS_INTERFACE_CONFIG:
+            raise dbus.exceptions.DBusException(
+                "org.freedesktop.DBus.Error.UnknownInterface: "
+                "FirewallD does not implement %s" % interface_name)
+
+        return {
+            'DefaultZone': self._get_property("DefaultZone"),
+            'MinimalMark': self._get_property("MinimalMark"),
+            'CleanupOnExit': self._get_property("CleanupOnExit"),
+            'Lockdown': self._get_property("Lockdown"),
+        }
+
+    @slip.dbus.polkit.require_auth(PK_ACTION_CONFIG)
+    @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='ssv')
+    @dbus_handle_exceptions
+    def Set(self, interface_name, property_name, new_value, sender=None):
+        interface_name = dbus_to_python(interface_name)
+        property_name = dbus_to_python(property_name)
+        new_value = dbus_to_python(new_value)
+        log.debug1("config.Set('%s', '%s', '%s')", interface_name,
+                   property_name, new_value)
+        self.accessCheck(sender)
+
+        if interface_name != DBUS_INTERFACE_CONFIG:
+            raise dbus.exceptions.DBusException(
+                "org.freedesktop.DBus.Error.UnknownInterface: "
+                "FirewallD does not implement %s" % interface_name)
+
+        if property_name in [ "DefaultZone", "MinimalMark", "CleanupOnExit",
+                              "Lockdown" ]:
+            if property_name == "MinimalMark":
+                try:
+                    foo = int(new_value)
+                except:
+                    raise FirewallError(INVALID_MARK, new_value)
+            try:
+                new_value = str(new_value)
+            except:
+                raise FirewallError(INVALID_VALUE, "'%s' for %s" % \
+                                            (new_value, property_name))
+            if property_name in [ "CleanupOnExit", "Lockdown" ]:
+                if new_value.lower() not in [ "yes", "no", "true", "false" ]:
+                    raise FirewallError(INVALID_VALUE, "'%s' for %s" % \
+                                            (new_value, property_name))
+            self.config.get_firewalld_conf().set(property_name, new_value)
+            self.config.get_firewalld_conf().write()
+            self.PropertiesChanged(interface_name,
+                                   { property_name: new_value }, [ ])
+        else:
+            raise dbus.exceptions.DBusException(
+                "org.freedesktop.DBus.Error.AccessDenied: "
+                "Property '%s' does not exist" % prop)
+
+    @dbus.service.signal(dbus.PROPERTIES_IFACE, signature='sa{sv}as')
+    def PropertiesChanged(self, interface_name, changed_properties,
+                          invalidated_properties):
+        pass
+
+    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
+
+    # policies
+
+    @dbus_service_method(DBUS_INTERFACE_CONFIG_POLICIES,
+                         out_signature=LockdownWhitelist.DBUS_SIGNATURE)
+    @dbus_handle_exceptions
+    def getLockdownWhitelist(self, sender=None):
+        log.debug1("config.getLockdownWhitelist()")
+        return self.config.get_policies().lockdown_whitelist.export_config()
+
+    @dbus_service_method(DBUS_INTERFACE_CONFIG_POLICIES, 
+                         in_signature=LockdownWhitelist.DBUS_SIGNATURE)
+    @dbus_handle_exceptions
+    def setLockdownWhitelist(self, settings, sender=None):
+        log.debug1("config.setLockdownWhitelistSettings(...)")
+        settings = dbus_to_python(settings)
+        self.config.get_policies().lockdown_whitelist.import_config(settings)
+        self.config.get_policies().lockdown_whitelist.write()
+        self.LockdownWhitelistUpdated()
+
+    @dbus.service.signal(DBUS_INTERFACE_CONFIG_POLICIES,)
+    @dbus_handle_exceptions
+    def LockdownWhitelistUpdated(self):
+        log.debug1("config.LockdownWhitelistUpdated()")
+
+    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
+
     # I C M P T Y P E S
 
     @dbus_service_method(DBUS_INTERFACE_CONFIG, out_signature='ao')
@@ -267,8 +419,9 @@ class FirewallDConfig(slip.dbus.service.Object):
                          out_signature='o')
     @dbus_handle_exceptions
     def getIcmpTypeByName(self, icmptype, sender=None):
-        """list icmptypes objects paths
+        """object path of icmptype with given name
         """
+        icmptype = dbus_to_python(icmptype)
         log.debug1("config.getIcmpTypeByName('%s')", icmptype)
         for obj in self.icmptypes:
             if obj.obj.name == icmptype:
@@ -280,12 +433,13 @@ class FirewallDConfig(slip.dbus.service.Object):
                          out_signature='o')
     @dbus_handle_exceptions
     def addIcmpType(self, icmptype, settings, sender=None):
-        """list icmptypes objects paths
+        """add icmptype with given name and settings
         """
-        icmptype = str(icmptype)
+        icmptype = dbus_to_python(icmptype)
+        settings = dbus_to_python(settings)
         log.debug1("config.addIcmpType('%s')", icmptype)
         self.accessCheck(sender)
-        obj = self.config.new_icmptype(icmptype, dbus_to_python(settings))
+        obj = self.config.new_icmptype(icmptype, settings)
         config_icmptype = self._addIcmpType(obj)
         return config_icmptype
 
@@ -308,8 +462,9 @@ class FirewallDConfig(slip.dbus.service.Object):
                          out_signature='o')
     @dbus_handle_exceptions
     def getServiceByName(self, service, sender=None):
-        """list services objects paths
+        """object path of service with given name
         """
+        service = dbus_to_python(service)
         log.debug1("config.getServiceByName('%s')", service)
         for obj in self.services:
             if obj.obj.name == service:
@@ -321,12 +476,13 @@ class FirewallDConfig(slip.dbus.service.Object):
                          out_signature='o')
     @dbus_handle_exceptions
     def addService(self, service, settings, sender=None):
-        """list services objects paths
+        """add service with given name and settings
         """
-        service = str(service)
+        service = dbus_to_python(service)
+        settings = dbus_to_python(settings)
         log.debug1("config.addService('%s')", service)
         self.accessCheck(sender)
-        obj = self.config.new_service(service, dbus_to_python(settings))
+        obj = self.config.new_service(service, settings)
         config_service = self._addService(obj)
         return config_service
 
@@ -349,25 +505,63 @@ class FirewallDConfig(slip.dbus.service.Object):
                          out_signature='o')
     @dbus_handle_exceptions
     def getZoneByName(self, zone, sender=None):
-        """list zones objects paths
+        """object path of zone with given name
         """
+        zone = dbus_to_python(zone)
         log.debug1("config.getZoneByName('%s')", zone)
         for obj in self.zones:
             if obj.obj.name == zone:
                 return obj
         raise FirewallError(INVALID_ZONE, zone)
 
+    @dbus_service_method(DBUS_INTERFACE_CONFIG, in_signature='s',
+                         out_signature='s')
+    @dbus_handle_exceptions
+    def getZoneOfInterface(self, iface, sender=None):
+        """name of zone the given interface belongs to
+        """
+        iface = dbus_to_python(iface)
+        log.debug1("config.getZoneOfInterface('%s')", iface)
+        ret = []
+        for obj in self.zones:
+            if iface in obj.obj.interfaces:
+                ret.append(obj.obj.name)
+        if len(ret) > 1:
+            # Even it shouldn't happen, it's actually possible that
+            # the same interface is in several zone XML files
+            return " ".join(ret) + "  (ERROR: interface '%s' is in %s zone XML files, can be only in one)" % (iface, len(ret))
+        return ret[0] if ret else ""
+
+    @dbus_service_method(DBUS_INTERFACE_CONFIG, in_signature='s',
+                         out_signature='s')
+    @dbus_handle_exceptions
+    def getZoneOfSource(self, source, sender=None):
+        """name of zone the given source belongs to
+        """
+        source = dbus_to_python(source)
+        log.debug1("config.getZoneOfSource('%s')", source)
+        ret = []
+        for obj in self.zones:
+            if source in obj.obj.sources:
+                ret.append(obj.obj.name)
+        if len(ret) > 1:
+            # Even it shouldn't happen, it's actually possible that
+            # the same source is in several zone XML files
+            return " ".join(ret) + "  (ERROR: source '%s' is in %s zone XML files, can be only in one)" % (iface, len(ret))
+        return ret[0] if ret else ""
+
     @dbus_service_method(DBUS_INTERFACE_CONFIG,
                          in_signature='s'+Zone.DBUS_SIGNATURE,
                          out_signature='o')
     @dbus_handle_exceptions
     def addZone(self, zone, settings, sender=None):
-        """list zones objects paths
+        """add zone with given name and settings
         """
-        zone = str(zone)
+        zone = dbus_to_python(zone)
+        settings = dbus_to_python(settings)
         log.debug1("config.addZone('%s')", zone)
         self.accessCheck(sender)
-        obj = self.config.new_zone(zone, dbus_to_python(settings))
+        obj = self.config.new_zone(zone, settings)
         config_zone = self._addZone(obj)
         return config_zone
 
